@@ -1,8 +1,8 @@
 /*****************************************************************************
- * beos_init.h: Initialization for BeOS specific features 
+ * beos_specific.h: BeOS specific features 
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: beos_specific.h,v 1.4 2001/03/21 13:42:33 sam Exp $
+ * $Id: beos_specific.h,v 1.7 2002/04/02 23:43:57 gbazin Exp $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *
@@ -29,9 +29,7 @@
 extern "C" {
 #endif
 
-void       beos_Create( void );
-void       beos_Destroy( void );
-char *     beos_GetProgramPath( void );
+char  * system_GetProgramPath( void );
 
 #ifdef __cplusplus
 }
