@@ -1,8 +1,8 @@
 /*****************************************************************************
  * input_ext-dec.h: structures exported to the VideoLAN decoders
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: input_ext-dec.h,v 1.41.2.2 2001/12/31 01:21:44 massiot Exp $
+ * Copyright (C) 1999-2001 VideoLAN
+ * $Id: input_ext-dec.h,v 1.54 2002/03/14 01:35:28 stef Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Michel Kaempf <maxx@via.ecp.fr>
@@ -40,25 +40,21 @@
  *****************************************************************************
  * Describe a data packet.
  *****************************************************************************/
-#define DATA_PACKET                                                         \
-    /* start of the PS or TS packet */                                      \
-    byte_t *                p_demux_start;                                  \
-    /* start of the PES payload in this packet */                           \
-    byte_t *                p_payload_start;                                \
-    byte_t *                p_payload_end; /* guess ? :-) */                \
-    /* is the packet messed up ? */                                         \
-    boolean_t               b_discard_payload;
-
 typedef struct data_packet_s
 {
     /* Used to chain the packets that carry data for a same PES or PSI */
     struct data_packet_s *  p_next;
 
-    DATA_PACKET
+    /* start of the PS or TS packet */
+    byte_t *                p_demux_start;
+    /* start of the PES payload in this packet */
+    byte_t *                p_payload_start;
+    byte_t *                p_payload_end; /* guess ? :-) */
+    /* is the packet messed up ? */
+    boolean_t               b_discard_payload;
 
-    /* Please note that at least one buffer allocator (in particular, the
-     * Next Generation Buffer Allocator) extends this structure with
-     * private data after DATA_PACKET. */
+    /* pointer to the real data */
+    struct data_buffer_s *  p_buffer;
 } data_packet_t;
 
 /*****************************************************************************
@@ -113,10 +109,8 @@ typedef struct decoder_fifo_s
     /* Communication interface between input and decoders */
     boolean_t               b_die;          /* the decoder should return now */
     boolean_t               b_error;      /* the decoder is in an error loop */
-    void *                  p_packets_mgt;   /* packets management services
-                                              * data (netlist...)            */
-    void                 (* pf_delete_pes)( void *, pes_packet_t * );
-                                     /* function to use when releasing a PES */
+    struct input_buffers_s *p_packets_mgt;   /* packets management services
+                                              * data */
 } decoder_fifo_t;
 
 /*****************************************************************************
@@ -158,9 +152,6 @@ typedef struct bit_stream_s
     /* The decoder fifo contains the data of the PES stream */
     decoder_fifo_t *        p_decoder_fifo;
 
-    /* Function to jump to the next data packet */
-    void                 (* pf_next_data_packet)( struct bit_stream_s * );
-
     /* Callback to the decoder used when changing data packets ; set
      * to NULL if your decoder doesn't need it. */
     void                 (* pf_bitstream_callback)( struct bit_stream_s *,
@@ -168,6 +159,12 @@ typedef struct bit_stream_s
     /* Optional argument to the callback */
     void *                  p_callback_arg;
 
+    /*
+     * PTS retrieval
+     */
+    mtime_t                 i_pts, i_dts;
+    byte_t *                p_pts_validity;
+
     /*
      * Byte structures
      */
@@ -212,9 +209,24 @@ typedef struct bit_stream_s
  * Prototypes from input_ext-dec.c
  *****************************************************************************/
 #ifndef PLUGIN
+void InitBitstream  ( struct bit_stream_s *, struct decoder_fifo_s *,
+                      void (* pf_bitstream_callback)( struct bit_stream_s *,
+                                                      boolean_t ),
+                      void * p_callback_arg );
+boolean_t NextDataPacket( struct decoder_fifo_s *, struct data_packet_s ** );
+void BitstreamNextDataPacket( struct bit_stream_s * );
 u32  UnalignedShowBits( struct bit_stream_s *, unsigned int );
 void UnalignedRemoveBits( struct bit_stream_s * );
 u32  UnalignedGetBits( struct bit_stream_s *, unsigned int );
+void CurrentPTS( struct bit_stream_s *, mtime_t *, mtime_t * );
+#else
+#   define InitBitstream p_symbols->InitBitstream
+#   define NextDataPacket p_symbols->NextDataPacket
+#   define BitstreamNextDataPacket p_symbols->BitstreamNextDataPacket
+#   define UnalignedShowBits p_symbols->UnalignedShowBits
+#   define UnalignedRemoveBits p_symbols->UnalignedRemoveBits
+#   define UnalignedGetBits p_symbols->UnalignedGetBits
+#   define CurrentPTS p_symbols->CurrentPTS
 #endif
 
 /*****************************************************************************
@@ -236,7 +248,7 @@ static __inline__ void AlignWord( bit_stream_t * p_bit_stream )
         }
         else
         {
-            p_bit_stream->pf_next_data_packet( p_bit_stream );
+            BitstreamNextDataPacket( p_bit_stream );
             p_bit_stream->fifo.buffer |= *(p_bit_stream->p_byte++)
                 << (8 * sizeof(WORD_TYPE) - 8
                      - p_bit_stream->fifo.i_available);
@@ -476,25 +488,25 @@ static __inline__ void GetChunk( bit_stream_t * p_bit_stream,
     if( (i_available = p_bit_stream->p_end - p_bit_stream->p_byte)
             >= i_buf_len )
     {
-        p_main->fast_memcpy( p_buffer, p_bit_stream->p_byte, i_buf_len );
+        FAST_MEMCPY( p_buffer, p_bit_stream->p_byte, i_buf_len );
         p_bit_stream->p_byte += i_buf_len;
     }
     else
     {
         do
         {
-            p_main->fast_memcpy( p_buffer, p_bit_stream->p_byte, i_available );
+            FAST_MEMCPY( p_buffer, p_bit_stream->p_byte, i_available );
             p_bit_stream->p_byte = p_bit_stream->p_end;
             p_buffer += i_available;
             i_buf_len -= i_available;
-            p_bit_stream->pf_next_data_packet( p_bit_stream );
+            BitstreamNextDataPacket( p_bit_stream );
         }
         while( (i_available = p_bit_stream->p_end - p_bit_stream->p_byte)
                 <= i_buf_len && !p_bit_stream->p_decoder_fifo->b_die );
 
         if( i_buf_len )
         {
-            p_main->fast_memcpy( p_buffer, p_bit_stream->p_byte, i_buf_len );
+            FAST_MEMCPY( p_buffer, p_bit_stream->p_byte, i_buf_len );
             p_bit_stream->p_byte += i_buf_len;
         }
     }
@@ -520,13 +532,9 @@ typedef struct decoder_config_s
     u16                     i_id;
     u8                      i_type;         /* type of the elementary stream */
 
+    void *                  p_demux_data;
     struct stream_ctrl_s *  p_stream_ctrl;
     struct decoder_fifo_s * p_decoder_fifo;
-    void                 (* pf_init_bit_stream)( struct bit_stream_s *,
-                                                 struct decoder_fifo_s *,
-                 void (* pf_bitstream_callback)( struct bit_stream_s *,
-                                                 boolean_t ),
-                                                 void * );
 } decoder_config_t;
 
 /*****************************************************************************
