@@ -3,7 +3,7 @@
  * Declaration and extern access to global program object.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: main.h,v 1.14 2001/03/21 13:42:33 sam Exp $
+ * $Id: main.h,v 1.33 2002/03/21 07:11:57 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -33,44 +33,50 @@
  * should only be used by interface thread.
  *****************************************************************************/
 
-typedef struct
+typedef struct main_s
 {
     /* Global properties */
     int                    i_argc;           /* command line arguments count */
     char **                ppsz_argv;              /* command line arguments */
-    char **                ppsz_env;                /* environment variables */
     char *                 psz_arg0;         /* program name (whithout path) */
+    char *                 psz_homedir;             /* user's home directory */
 
-    int                    i_cpu_capabilities;             /* CPU extensions */
+    u32                    i_cpu_capabilities;             /* CPU extensions */
     int                    i_warning_level;        /* warning messages level */
+    boolean_t              b_stats;                  /* display statistics ? */
 
     /* Generic settings */
     boolean_t              b_audio;             /* is audio output allowed ? */
     boolean_t              b_video;             /* is video output allowed ? */
-    boolean_t              b_vlans;                 /* are vlans supported ? */
-    boolean_t              b_dvd;                              /* DVD mode ? */
+    boolean_t              b_stereo;
+    mtime_t                i_desync;   /* relative desync of the audio ouput */
+
+    /* Fast memcpy plugin used */
+    module_t *             p_memcpy_module;
+    void* ( *pf_memcpy ) ( void *, const void *, size_t );
+    void* ( *pf_memset ) ( void *, int, size_t );    /* FIXME: unimplemented */
 
     /* Unique threads */
-    p_vout_thread_t        p_vout;                    /* video output thread */
-    p_aout_thread_t        p_aout;                    /* audio output thread */
     p_intf_thread_t        p_intf;                  /* main interface thread */
 
     /* Shared data - these structures are accessed directly from p_main by
      * several modules */
-    struct module_bank_s * p_bank;                            /* module bank */
     p_playlist_t           p_playlist;                           /* playlist */
     p_intf_msg_t           p_msg;                 /* messages interface data */
-    p_input_vlan_t         p_vlan;                      /* vlan library data */
+    p_input_channel_t      p_channel;                /* channel library data */
+
+    /* Locks */
+    vlc_mutex_t            config_lock;          /* lock for the config file */
 } main_t;
 
+#ifndef PLUGIN
 extern main_t *p_main;
+#else
+#   define p_main (p_symbols->p_main)
+#endif
 
 /*****************************************************************************
- * Prototypes - these methods are used to get default values for some threads
- * and modules.
+ * Fast memory operation module
  *****************************************************************************/
-int    main_GetIntVariable( char *psz_name, int i_default );
-char * main_GetPszVariable( char *psz_name, char *psz_default );
-void   main_PutIntVariable( char *psz_name, int i_value );
-void   main_PutPszVariable( char *psz_name, char *psz_value );
-
+#define FAST_MEMCPY p_main->pf_memcpy
+#define FAST_MEMSET p_main->pf_memset
