@@ -2,7 +2,7 @@
  * modules.h : Module management functions.
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: modules.h,v 1.35 2001/12/07 18:33:07 sam Exp $
+ * $Id: modules.h,v 1.46 2002/03/20 03:43:51 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -21,27 +21,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-/*****************************************************************************
- * module_bank_t, p_module_bank (global variable)
- *****************************************************************************
- * This global variable is accessed by any function using modules.
- *****************************************************************************/
-typedef struct
-{
-    struct module_s *   first; /* First module of the bank */
-
-    vlc_mutex_t         lock;  /* Global lock -- you can't imagine how awful it
-                                  is to design thread-safe linked lists. */
-} module_bank_t;
-
-extern module_bank_t *p_module_bank;
-
 /*****************************************************************************
  * Module #defines.
  *****************************************************************************/
 
 /* Number of tries before we unload an unused module */
 #define MODULE_HIDE_DELAY 10000
+#define MODULE_SHORTCUT_MAX 10
 
 /* The module handle type. */
 #ifdef SYS_BEOS
@@ -53,47 +39,142 @@ typedef void *  module_handle_t;
 /*****************************************************************************
  * Module capabilities.
  *****************************************************************************/
-
-#define MODULE_CAPABILITY_NULL     0        /* The Module can't do anything */
-#define MODULE_CAPABILITY_INTF     1 <<  0  /* Interface */
-#define MODULE_CAPABILITY_ACCESS   1 <<  1  /* Input */
-#define MODULE_CAPABILITY_INPUT    1 <<  2  /* Input */
-#define MODULE_CAPABILITY_DECAPS   1 <<  3  /* Decaps */
-#define MODULE_CAPABILITY_DEC      1 <<  4  /* Video decoder */
-#define MODULE_CAPABILITY_MOTION   1 <<  5  /* Motion compensation */
-#define MODULE_CAPABILITY_IDCT     1 <<  6  /* IDCT transformation */
-#define MODULE_CAPABILITY_AOUT     1 <<  7  /* Audio output */
-#define MODULE_CAPABILITY_VOUT     1 <<  8  /* Video output */
-#define MODULE_CAPABILITY_YUV      1 <<  9  /* YUV colorspace conversion */
-#define MODULE_CAPABILITY_IMDCT    1 << 10  /* IMDCT transformation */
-#define MODULE_CAPABILITY_DOWNMIX  1 << 11  /* AC3 downmix */
-#define MODULE_CAPABILITY_MEMCPY   1 << 12  /* memcpy */
-
-/* FIXME: kludge */
-struct input_area_s;
-struct imdct_s;
-struct complex_s;
-struct dm_par_s;
-struct bit_stream_s;
-struct decoder_fifo_s;
-
-struct decoder_config_s;
-
-/* FIXME: not yet used */
-typedef struct probedata_s
+static __inline__ char *GetCapabilityName( unsigned int i_capa )
 {
-    u8  i_type;
-    struct
+    /* The sole purpose of this inline function and the ugly #defines
+     * around it is to avoid having two places to modify when adding a
+     * new capability. */
+    static char *pp_capa[] =
     {
-        char * psz_data;
-    } aout;
-} probedata_t;
+        "main",
+#define MODULE_CAPABILITY_MAIN      0  /* Main */
+        "interface",
+#define MODULE_CAPABILITY_INTF      1  /* Interface */
+        "access",
+#define MODULE_CAPABILITY_ACCESS    2  /* Input */
+        "demux",
+#define MODULE_CAPABILITY_DEMUX     3  /* Input */
+        "network",
+#define MODULE_CAPABILITY_NETWORK   4  /* Network */
+        "decoder",
+#define MODULE_CAPABILITY_DECODER   5  /* Audio or video decoder */
+        "motion",
+#define MODULE_CAPABILITY_MOTION    6  /* Motion compensation */
+        "iDCT",
+#define MODULE_CAPABILITY_IDCT      7  /* IDCT transformation */
+        "audio output",
+#define MODULE_CAPABILITY_AOUT      8  /* Audio output */
+        "video output",
+#define MODULE_CAPABILITY_VOUT      9  /* Video output */
+        "chroma transformation",
+#define MODULE_CAPABILITY_CHROMA   10  /* colorspace conversion */
+        "iMDCT",
+#define MODULE_CAPABILITY_IMDCT    11  /* IMDCT transformation */
+        "downmix",
+#define MODULE_CAPABILITY_DOWNMIX  12  /* AC3 downmix */
+        "memcpy",
+#define MODULE_CAPABILITY_MEMCPY   13  /* memcpy */
+        "unknown"
+#define MODULE_CAPABILITY_MAX      14  /* Total number of capabilities */
+    };
 
-/* FIXME: find a nicer way to do this. */
+    return pp_capa[ (i_capa) > MODULE_CAPABILITY_MAX ? MODULE_CAPABILITY_MAX :
+                    (i_capa) ];
+}
+
+/*****************************************************************************
+ * module_bank_t, p_module_bank (global variable)
+ *****************************************************************************
+ * This global variable is accessed by any function using modules.
+ *****************************************************************************/
+typedef struct module_bank_s
+{
+    struct module_s *   first;                   /* First module in the bank */
+    int                 i_count;              /* Number of allocated modules */
+
+    vlc_mutex_t         lock;  /* Global lock -- you can't imagine how awful *
+                                    it is to design thread-safe linked lists */
+} module_bank_t;
+
+#ifndef PLUGIN
+extern module_bank_t *p_module_bank;
+#else
+#   define p_module_bank (p_symbols->p_module_bank)
+#endif
+
+/*****************************************************************************
+ * Module description structure
+ *****************************************************************************/
+typedef struct module_s
+{
+    /*
+     * Variables set by the module to identify itself
+     */
+    char *psz_name;                                  /* Module _unique_ name */
+    char *psz_longname;                           /* Module descriptive name */
+
+    /*
+     * Variables set by the module to tell us what it can do
+     */
+    char *psz_program;        /* Program name which will activate the module */
+    char *pp_shortcuts[ MODULE_SHORTCUT_MAX ];    /* Shortcuts to the module */
+
+    u32   i_capabilities;                                 /* Capability list */
+    int   pi_score[ MODULE_CAPABILITY_MAX ];    /* Score for each capability */
+
+    u32   i_cpu_capabilities;                   /* Required CPU capabilities */
+
+    struct module_functions_s *p_functions;          /* Capability functions */
+
+    /*
+     * Variables set by the module to store its config options
+     */
+    struct module_config_s *p_config;      /* Module configuration structure */
+    struct module_config_s *p_config_orig;    /* original module config data */
+    vlc_mutex_t            config_lock;    /* lock used to modify the config */
+    unsigned int           i_config_lines;  /* number of configuration lines */
+    unsigned int           i_config_items;  /* number of configuration items */
+
+    /*
+     * Variables used internally by the module manager
+     */
+    boolean_t           b_builtin;  /* Set to true if the module is built in */
+
+    union
+    {
+        struct
+        {
+            module_handle_t     handle;                     /* Unique handle */
+            char *              psz_filename;             /* Module filename */
+
+        } plugin;
+
+        struct
+        {
+            int ( *pf_deactivate ) ( struct module_s * );
+
+        } builtin;
+
+    } is;
+
+    int   i_usage;                                      /* Reference counter */
+    int   i_unused_delay;                  /* Delay until module is unloaded */
+
+    struct module_s *next;                                    /* Next module */
+    struct module_s *prev;                                /* Previous module */
+
+    /*
+     * Symbol table we send to the module so that it can access vlc symbols
+     */
+    struct module_symbols_s *p_symbols;
+
+} module_t;
+
+/*****************************************************************************
+ * Module functions description structure
+ *****************************************************************************/
 typedef struct function_list_s
 {
-    int ( * pf_probe ) ( probedata_t * p_data );
-
     union
     {
         /* Interface plugin */
@@ -104,45 +185,40 @@ typedef struct function_list_s
             void ( * pf_run )  ( struct intf_thread_s * );
         } intf;
 
-        /* Input plugin */
+        /* Access plugin */
         struct
         {
-            void ( * pf_init ) ( struct input_thread_s * );
-            void ( * pf_open ) ( struct input_thread_s * );
+            int  ( * pf_open ) ( struct input_thread_s * );
             void ( * pf_close )( struct input_thread_s * );
-            void ( * pf_end )  ( struct input_thread_s * );
-            void ( * pf_init_bit_stream ) ( struct bit_stream_s *,
-                                            struct decoder_fifo_s *,
-                      void (* pf_bitstream_callback)( struct bit_stream_s *,
-                                                      boolean_t ),
-                                           void * );
-
-            int  ( * pf_read ) ( struct input_thread_s *,
-                                 struct data_packet_s *
-                                        pp_packets[] );
-            void ( * pf_demux )( struct input_thread_s *,
-                                 struct data_packet_s * );
-
-            struct data_packet_s * ( * pf_new_packet ) ( void *, size_t );
-            struct pes_packet_s *  ( * pf_new_pes )    ( void * );
-            void ( * pf_delete_packet )  ( void *, struct data_packet_s * );
-            void ( * pf_delete_pes )     ( void *, struct pes_packet_s * );
-
+            ssize_t ( * pf_read ) ( struct input_thread_s *, byte_t *, size_t );
+            void ( * pf_seek ) ( struct input_thread_s *, off_t );
             int  ( * pf_set_program ) ( struct input_thread_s *,
-                                     struct pgrm_descriptor_s * );
-
+                                        struct pgrm_descriptor_s * );
             int  ( * pf_set_area ) ( struct input_thread_s *,
                                      struct input_area_s * );
+        } access;
+
+        /* Demux plugin */
+        struct
+        {
+            int  ( * pf_init ) ( struct input_thread_s * );
+            void ( * pf_end )  ( struct input_thread_s * );
+            int  ( * pf_demux )( struct input_thread_s * );
             int  ( * pf_rewind )   ( struct input_thread_s * );
-            void ( * pf_seek )     ( struct input_thread_s *, off_t );
-        } input;
+        } demux;
+
+        /* Network plugin */
+        struct
+        {
+            int  ( * pf_open )( struct network_socket_s * );
+        } network;
 
         /* Audio output plugin */
         struct
         {
             int  ( * pf_open )       ( struct aout_thread_s * );
             int  ( * pf_setformat )  ( struct aout_thread_s * );
-            long ( * pf_getbufinfo ) ( struct aout_thread_s *, long );
+            int  ( * pf_getbufinfo ) ( struct aout_thread_s *, int );
             void ( * pf_play )       ( struct aout_thread_s *, byte_t *, int );
             void ( * pf_close )      ( struct aout_thread_s * );
         } aout;
@@ -155,9 +231,10 @@ typedef struct function_list_s
             void ( * pf_end )        ( struct vout_thread_s * );
             void ( * pf_destroy )    ( struct vout_thread_s * );
             int  ( * pf_manage )     ( struct vout_thread_s * );
-            void ( * pf_display )    ( struct vout_thread_s * );
-            void ( * pf_setpalette ) ( struct vout_thread_s *, u16 *red,
-                                       u16 *green, u16 *blue, u16 *transp );
+            void ( * pf_render )     ( struct vout_thread_s *,
+                                       struct picture_s * );
+            void ( * pf_display )    ( struct vout_thread_s *,
+                                       struct picture_s * );
         } vout;
 
         /* Motion compensation plugin */
@@ -182,13 +259,12 @@ typedef struct function_list_s
             void ( * pf_norm_scan )    ( u8 ppi_scan[2][64] );
         } idct;
 
-        /* YUV transformation plugin */
+        /* Chroma transformation plugin */
         struct
         {
             int  ( * pf_init )         ( struct vout_thread_s * );
-            int  ( * pf_reset )        ( struct vout_thread_s * );
             void ( * pf_end )          ( struct vout_thread_s * );
-        } yuv;
+        } chroma;
 
         /* IMDCT plugin */
         struct
@@ -222,13 +298,15 @@ typedef struct function_list_s
         /* Decoder plugins */
         struct
         {
+            int  ( * pf_probe)( u8 * p_es );
             int  ( * pf_run ) ( struct decoder_config_s * p_config );
         } dec;
 
         /* memcpy plugins */
         struct
         {
-            void* ( * fast_memcpy ) ( void *, const void *, size_t );
+            void* ( * pf_memcpy ) ( void *, const void *, size_t );
+            void* ( * pf_memset ) ( void *, int, size_t );
         } memcpy;
 
     } functions;
@@ -240,14 +318,14 @@ typedef struct module_functions_s
     /* XXX: The order here has to be the same as above for the #defines */
     function_list_t intf;
     function_list_t access;
-    function_list_t input;
-    function_list_t decaps;
+    function_list_t demux;
+    function_list_t network;
     function_list_t dec;
     function_list_t motion;
     function_list_t idct;
     function_list_t aout;
     function_list_t vout;
-    function_list_t yuv;
+    function_list_t chroma;
     function_list_t imdct;
     function_list_t downmix;
     function_list_t memcpy;
@@ -256,87 +334,21 @@ typedef struct module_functions_s
 
 typedef struct module_functions_s * p_module_functions_t;
 
-/*****************************************************************************
- * Macros used to build the configuration structure.
- *****************************************************************************/
-
-/* Mandatory first and last parts of the structure */
-#define MODULE_CONFIG_ITEM_START       0xdead  /* The main window */
-#define MODULE_CONFIG_ITEM_END         0xbeef  /* End of the window */
-
-/* Configuration widgets */
-#define MODULE_CONFIG_ITEM_WINDOW      0x0001  /* The main window */
-#define MODULE_CONFIG_ITEM_PANE        0x0002  /* A notebook pane */
-#define MODULE_CONFIG_ITEM_FRAME       0x0003  /* A frame */
-#define MODULE_CONFIG_ITEM_COMMENT     0x0004  /* A comment text */
-#define MODULE_CONFIG_ITEM_STRING      0x0005  /* A string */
-#define MODULE_CONFIG_ITEM_FILE        0x0006  /* A file selector */
-#define MODULE_CONFIG_ITEM_CHECK       0x0007  /* A checkbox */
-#define MODULE_CONFIG_ITEM_CHOOSE      0x0008  /* A choose box */
-#define MODULE_CONFIG_ITEM_RADIO       0x0009  /* A radio box */
-#define MODULE_CONFIG_ITEM_SCALE       0x000a  /* A horizontal ruler */
-#define MODULE_CONFIG_ITEM_SPIN        0x000b  /* A numerical selector */
-
-typedef struct module_config_s
-{
-    int         i_type;                         /* Configuration widget type */
-    char *      psz_text;        /* Text commenting or describing the widget */
-    char *      psz_name;                                   /* Variable name */
-    void *      p_getlist;          /* Function to call to get a choice list */
-    void *      p_change;        /* Function to call when commiting a change */
-} module_config_t;
-
-/*****************************************************************************
- * Bank and module description structures
- *****************************************************************************/
-
-/* The module description structure */
-typedef struct module_s
-{
-    boolean_t           b_builtin;  /* Set to true if the module is built in */
-
-    union
-    {
-        struct
-        {
-            module_handle_t     handle;                     /* Unique handle */
-            char *              psz_filename;             /* Module filename */
-
-        } plugin;
-
-        struct
-        {
-            int ( *pf_deactivate ) ( struct module_s * );
-
-        } builtin;
-
-    } is;
-
-    char *              psz_name;                    /* Module _unique_ name */
-    char *              psz_longname;             /* Module descriptive name */
-    char *              psz_version;                       /* Module version */
-
-    int                 i_usage;                        /* Reference counter */
-    int                 i_unused_delay;    /* Delay until module is unloaded */
-
-    struct module_s *   next;                                 /* Next module */
-    struct module_s *   prev;                             /* Previous module */
-
-    module_config_t         *p_config;     /* Module configuration structure */
-    struct module_symbols_s *p_symbols;
-
-    u32                      i_capabilities;              /* Capability list */
-    p_module_functions_t     p_functions;            /* Capability functions */
-
-} module_t;
-
 /*****************************************************************************
  * Exported functions.
  *****************************************************************************/
+#ifndef PLUGIN
 void            module_InitBank     ( void );
+void            module_LoadMain     ( void );
+void            module_LoadBuiltins ( void );
+void            module_LoadPlugins  ( void );
 void            module_EndBank      ( void );
 void            module_ResetBank    ( void );
 void            module_ManageBank   ( void );
-module_t *      module_Need         ( int i_capabilities, void *p_data );
+module_t *      module_Need         ( int, char *, void * );
 void            module_Unneed       ( module_t * p_module );
 
+#else
+#   define module_Need p_symbols->module_Need
+#   define module_Unneed p_symbols->module_Unneed
+#endif
