@@ -9,7 +9,7 @@
  * Functions prototyped are implemented in interface/mtime.c.
  *****************************************************************************
  * Copyright (C) 1996, 1997, 1998, 1999, 2000 VideoLAN
- * $Id: mtime.h,v 1.8 2001/11/28 15:08:04 massiot Exp $
+ * $Id: mtime.h,v 1.10 2002/01/29 20:11:18 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -28,12 +28,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-/*****************************************************************************
- * Required headers:
- *  none
- * this header includes inline functions
- *****************************************************************************/
-
 /*****************************************************************************
  * LAST_MDATE: date which will never happen
  *****************************************************************************
@@ -56,7 +50,14 @@
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
+#ifndef PLUGIN
 char *  mstrtime ( char *psz_buffer, mtime_t date );
 mtime_t mdate    ( void );
 void    mwait    ( mtime_t date );
 void    msleep   ( mtime_t delay );
+#else
+#   define msleep    p_symbols->msleep
+#   define mdate     p_symbols->mdate
+#   define mstrtime  p_symbols->mstrtime
+#endif
+
