@@ -2,7 +2,7 @@
  * InterfaceWindow.cpp: beos interface
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001 VideoLAN
- * $Id: InterfaceWindow.cpp,v 1.10 2001/12/07 18:33:07 sam Exp $
+ * $Id: InterfaceWindow.cpp,v 1.15 2002/03/31 08:13:38 tcastley Exp $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -23,7 +23,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
-#include "defs.h"
 
 /* System headers */
 #include <kernel/OS.h>
@@ -40,15 +39,14 @@
 /* VLC headers */
 extern "C"
 {
-#include "common.h"
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"
+#include <videolan/vlc.h>
+
 #include "stream_control.h"
 #include "input_ext-intf.h"
+
 #include "interface.h"
 #include "intf_playlist.h"
+
 #include "audio_output.h"
 }
 
@@ -86,6 +84,7 @@ InterfaceWindow::InterfaceWindow( BRect frame, const char *name,
     BMenu *mAudio;
     CDMenu *cd_menu;
     BMenu *mNavigation;
+    BMenu *mConfig;
     
     /* Add the file Menu */
     BMenuItem *mItem;
@@ -126,6 +125,12 @@ InterfaceWindow::InterfaceWindow( BRect frame, const char *name,
     mNavigation->AddItem( new BMenuItem( "Next Chapter",
                                         new BMessage(NEXT_CHAPTER)) );
                                         
+    /* Add the Config menu */
+    menu_bar->AddItem( mConfig = new BMenu( "Config" ) );
+    menu_bar->ResizeToPreferred();
+    mConfig->AddItem( miOnTop = new BMenuItem( "Always on Top",
+                                        new BMessage(TOGGLE_ON_TOP)) );
+    miOnTop->SetMarked(false);                                    
 
     ResizeTo(260,50 + menu_bar->Bounds().IntegerHeight()+1);
     controlRect = Bounds();
@@ -133,6 +138,8 @@ InterfaceWindow::InterfaceWindow( BRect frame, const char *name,
 
     p_mediaControl = new MediaControlView( controlRect );
     p_mediaControl->SetViewColor( ui_color(B_PANEL_BACKGROUND_COLOR) );
+    b_empty_playlist = true;
+    p_mediaControl->SetEnabled( !b_empty_playlist );
 
     /* Show */
     AddChild( p_mediaControl );
@@ -155,9 +162,9 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
     BAlert *alert;
 
     Activate();
-    if (p_intf->p_input)
+    if (p_input_bank->pp_input[0])
     {
-	    playback_status = p_intf->p_input->stream.control.i_status;
+	    playback_status = p_input_bank->pp_input[0]->stream.control.i_status;
 	}
 	else
 	{
@@ -171,6 +178,20 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
         alert->Go();
         break;
 
+    case TOGGLE_ON_TOP:
+        miOnTop->SetMarked(! miOnTop->IsMarked() );
+        if ( miOnTop->IsMarked() )
+        {
+            SetFeel(B_FLOATING_ALL_WINDOW_FEEL);
+            SetWorkspaces(B_CURRENT_WORKSPACE); 
+        }
+        else
+        {
+            SetFeel(B_NORMAL_WINDOW_FEEL);
+            SetWorkspaces(B_CURRENT_WORKSPACE); 
+        }
+        break;
+        
     case OPEN_FILE:
         if( file_panel )
         {
@@ -180,6 +201,8 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
         file_panel = new BFilePanel();
         file_panel->SetTarget( this );
         file_panel->Show();
+        b_empty_playlist = false;
+        p_mediaControl->SetEnabled( !b_empty_playlist );
         break;
 
 	case OPEN_PLAYLIST:
@@ -199,18 +222,20 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
                       "dvd:%s", psz_device );
             psz_source[ strlen(psz_source) ] = '\0';
             intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END, (char*)psz_source );
-            if( p_intf->p_input != NULL )
+            if( p_input_bank->pp_input[0] != NULL )
             {
-                p_intf->p_input->b_eof = 1;
+                p_input_bank->pp_input[0]->b_eof = 1;
             }
             intf_PlaylistJumpto( p_main->p_playlist, 
                                  p_main->p_playlist->i_size - 1 );
+            b_empty_playlist = false;
+            p_mediaControl->SetEnabled( !b_empty_playlist );
         }
         break;
 
     case STOP_PLAYBACK:
         // this currently stops playback not nicely
-        if( p_intf->p_input != NULL )
+        if( p_input_bank->pp_input[0] != NULL )
         {
             // silence the sound, otherwise very horrible
             vlc_mutex_lock( &p_aout_bank->lock );
@@ -223,7 +248,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             snooze( 400000 );
 
             /* end playing item */
-            p_intf->p_input->b_eof = 1;
+            p_input_bank->pp_input[0]->b_eof = 1;
             
             /* update playlist */
             vlc_mutex_lock( &p_main->p_playlist->change_lock );
@@ -239,7 +264,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
 
     case PAUSE_PLAYBACK:
         /* toggle between pause and play */
-        if( p_intf->p_input != NULL )
+        if( p_input_bank->pp_input[0] != NULL )
         {
             /* pause if currently playing */
             if( playback_status == PLAYING_S )
@@ -256,7 +281,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
                 snooze( 400000 );
                 
                 /* pause the movie */
-                input_SetStatus( p_intf->p_input, INPUT_STATUS_PAUSE );
+                input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PAUSE );
                 vlc_mutex_lock( &p_main->p_playlist->change_lock );
                 p_main->p_playlist->b_stopped = 0;
                 vlc_mutex_unlock( &p_main->p_playlist->change_lock );
@@ -276,7 +301,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
                 snooze( 400000 );
                 
                 /* Start playing */
-                input_SetStatus( p_intf->p_input, INPUT_STATUS_PLAY );
+                input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );
                 p_main->p_playlist->b_stopped = 0;
             }
         }
@@ -303,7 +328,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
 
     case FASTER_PLAY:
         /* cycle the fast playback modes */
-        if( p_intf->p_input != NULL )
+        if( p_input_bank->pp_input[0] != NULL )
         {
             /* mute the sound */
             vlc_mutex_lock( &p_aout_bank->lock );
@@ -317,7 +342,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             snooze( 400000 );
 
             /* change the fast play mode */
-            input_SetStatus( p_intf->p_input, INPUT_STATUS_FASTER );
+            input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_FASTER );
             vlc_mutex_lock( &p_main->p_playlist->change_lock );
             p_main->p_playlist->b_stopped = 0;
             vlc_mutex_unlock( &p_main->p_playlist->change_lock );
@@ -326,7 +351,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
 
     case SLOWER_PLAY:
         /*  cycle the slow playback modes */
-        if (p_intf->p_input != NULL )
+        if (p_input_bank->pp_input[0] != NULL )
         {
             /* mute the sound */
             vlc_mutex_lock( &p_aout_bank->lock );
@@ -340,7 +365,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             snooze( 400000 );
 
             /* change the slower play */
-            input_SetStatus( p_intf->p_input, INPUT_STATUS_SLOWER );
+            input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_SLOWER );
             vlc_mutex_lock( &p_main->p_playlist->change_lock );
             p_main->p_playlist->b_stopped = 0;
             vlc_mutex_unlock( &p_main->p_playlist->change_lock );
@@ -394,12 +419,12 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             int32 i = p_message->FindInt32( "channel" );
             if ( i == -1 )
             {
-                input_ChangeES( p_intf->p_input, NULL, AUDIO_ES );
+                input_ChangeES( p_input_bank->pp_input[0], NULL, AUDIO_ES );
             }
             else
             {
-                input_ChangeES( p_intf->p_input,
-                        p_intf->p_input->stream.pp_es[i], AUDIO_ES );
+                input_ChangeES( p_input_bank->pp_input[0],
+                        p_input_bank->pp_input[0]->stream.pp_es[i], AUDIO_ES );
             }
         }
         break;
@@ -409,12 +434,12 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             int32 i = p_message->FindInt32( "subtitle" );
             if ( i == -1 )
             {
-                input_ChangeES( p_intf->p_input, NULL, SPU_ES);
+                input_ChangeES( p_input_bank->pp_input[0], NULL, SPU_ES);
             }
             else
             {
-                input_ChangeES( p_intf->p_input,
-                        p_intf->p_input->stream.pp_es[i], SPU_ES );
+                input_ChangeES( p_input_bank->pp_input[0],
+                        p_input_bank->pp_input[0]->stream.pp_es[i], SPU_ES );
             }
         }
         break;
@@ -423,14 +448,14 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             input_area_t *  p_area;
             int             i_id;
 
-            i_id = p_intf->p_input->stream.p_selected_area->i_id - 1;
+            i_id = p_input_bank->pp_input[0]->stream.p_selected_area->i_id - 1;
 
             /* Disallow area 0 since it is used for video_ts.vob */
             if( i_id > 0 )
             {
-                p_area = p_intf->p_input->stream.pp_areas[i_id];
-                input_ChangeArea( p_intf->p_input, (input_area_t*)p_area );
-                input_SetStatus( p_intf->p_input, INPUT_STATUS_PLAY );
+                p_area = p_input_bank->pp_input[0]->stream.pp_areas[i_id];
+                input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );
+                input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );
             }
             break;
         }
@@ -439,13 +464,13 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             input_area_t *  p_area;
             int             i_id;
 
-            i_id = p_intf->p_input->stream.p_selected_area->i_id + 1;
+            i_id = p_input_bank->pp_input[0]->stream.p_selected_area->i_id + 1;
 
-            if( i_id < p_intf->p_input->stream.i_area_nb )
+            if( i_id < p_input_bank->pp_input[0]->stream.i_area_nb )
             {
-                p_area = p_intf->p_input->stream.pp_areas[i_id];
-                input_ChangeArea( p_intf->p_input, (input_area_t*)p_area );
-                input_SetStatus( p_intf->p_input, INPUT_STATUS_PLAY );
+                p_area = p_input_bank->pp_input[0]->stream.pp_areas[i_id];
+                input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );
+                input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );
             }
         }
         break;
@@ -453,13 +478,13 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
         {
             input_area_t *  p_area;
 
-            p_area = p_intf->p_input->stream.p_selected_area;
+            p_area = p_input_bank->pp_input[0]->stream.p_selected_area;
 
             if( p_area->i_part > 0 )
             {
                 p_area->i_part--;
-                input_ChangeArea( p_intf->p_input, (input_area_t*)p_area );
-                input_SetStatus( p_intf->p_input, INPUT_STATUS_PLAY );
+                input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );
+                input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );
             }
         }
         break;
@@ -467,13 +492,13 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
         {
             input_area_t *  p_area;
 
-            p_area = p_intf->p_input->stream.p_selected_area;
+            p_area = p_input_bank->pp_input[0]->stream.p_selected_area;
 
             if( p_area->i_part > 0 )
             {
                 p_area->i_part++;
-                input_ChangeArea( p_intf->p_input, (input_area_t*)p_area );
-                input_SetStatus( p_intf->p_input, INPUT_STATUS_PLAY );
+                input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );
+                input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );
             }
         }
         break;
@@ -486,9 +511,9 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
                 BPath path( &ref );
                 intf_PlaylistAdd( p_main->p_playlist,
                                   PLAYLIST_END, (char*)path.Path() );
-                if( p_intf->p_input != NULL )
+                if( p_input_bank->pp_input[0] != NULL )
                 {
-                    p_intf->p_input->b_eof = 1;
+                    p_input_bank->pp_input[0]->b_eof = 1;
                 }
                 intf_PlaylistJumpto( p_main->p_playlist, 
                                      p_main->p_playlist->i_size - 1 );
@@ -509,20 +534,20 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
  *****************************************************************************/
 void InterfaceWindow::updateInterface()
 {
-	if ( p_intf->p_input )
+	if ( p_input_bank->pp_input[0] )
 	{
         if ( acquire_sem(p_mediaControl->fScrubSem) == B_OK )
         {
             uint64 seekTo = (p_mediaControl->GetSeekTo() *
-                        p_intf->p_input->stream.p_selected_area->i_size) / 100;
-            input_Seek( p_intf->p_input, seekTo);
+                        p_input_bank->pp_input[0]->stream.p_selected_area->i_size) / 100;
+            input_Seek( p_input_bank->pp_input[0], seekTo);
         }
         else if( Lock() )
         {
-            p_mediaControl->SetStatus(p_intf->p_input->stream.control.i_status, 
-                                      p_intf->p_input->stream.control.i_rate);
-            p_mediaControl->SetProgress(p_intf->p_input->stream.p_selected_area->i_tell,
-                                        p_intf->p_input->stream.p_selected_area->i_size);
+            p_mediaControl->SetStatus(p_input_bank->pp_input[0]->stream.control.i_status, 
+                                      p_input_bank->pp_input[0]->stream.control.i_rate);
+            p_mediaControl->SetProgress(p_input_bank->pp_input[0]->stream.p_selected_area->i_tell,
+                                        p_input_bank->pp_input[0]->stream.p_selected_area->i_size);
             Unlock();
         }
     }
@@ -686,26 +711,26 @@ int LanguageMenu::GetChannels()
     AddItem(menu_item);
     menu_item->SetMarked(TRUE);
 
-    if( p_intf->p_input == NULL )
+    if( p_input_bank->pp_input[0] == NULL )
     {
         return 1;
     }
 
 
-    vlc_mutex_lock( &p_intf->p_input->stream.stream_lock );
-    for( i = 0; i < p_intf->p_input->stream.i_selected_es_number; i++ )
+    vlc_mutex_lock( &p_input_bank->pp_input[0]->stream.stream_lock );
+    for( i = 0; i < p_input_bank->pp_input[0]->stream.i_selected_es_number; i++ )
     {
-        if( kind == p_intf->p_input->stream.pp_selected_es[i]->i_cat )
+        if( kind == p_input_bank->pp_input[0]->stream.pp_selected_es[i]->i_cat )
         {
-            p_es = p_intf->p_input->stream.pp_selected_es[i];
+            p_es = p_input_bank->pp_input[0]->stream.pp_selected_es[i];
         }
     }
 
-    for( i = 0; i < p_intf->p_input->stream.i_es_number; i++ )
+    for( i = 0; i < p_input_bank->pp_input[0]->stream.i_es_number; i++ )
     {
-        if( kind == p_intf->p_input->stream.pp_es[i]->i_cat )
+        if( kind == p_input_bank->pp_input[0]->stream.pp_es[i]->i_cat )
         {
-            psz_name = p_intf->p_input->stream.pp_es[i]->psz_desc;
+            psz_name = p_input_bank->pp_input[0]->stream.pp_es[i]->psz_desc;
             if( kind == AUDIO_ES ) //audio
             {
                 msg = new BMessage(SELECT_CHANNEL);
@@ -719,11 +744,11 @@ int LanguageMenu::GetChannels()
             BMenuItem *menu_item;
             menu_item = new BMenuItem(psz_name, msg);
             AddItem(menu_item);
-            b_active = (p_es == p_intf->p_input->stream.pp_es[i]);
+            b_active = (p_es == p_input_bank->pp_input[0]->stream.pp_es[i]);
             menu_item->SetMarked(b_active);
         }
     }
-    vlc_mutex_unlock( &p_intf->p_input->stream.stream_lock );
+    vlc_mutex_unlock( &p_input_bank->pp_input[0]->stream.stream_lock );
 
 }
 
