@@ -2,7 +2,7 @@
  * vout_beos.cpp: beos video output display method
  *****************************************************************************
  * Copyright (C) 2000, 2001 VideoLAN
- * $Id: vout_beos.cpp,v 1.31 2001/10/21 06:06:20 tcastley Exp $
+ * $Id: vout_beos.cpp,v 1.51 2002/04/02 10:44:40 tcastley Exp $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -24,54 +24,31 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME beos
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <errno.h>                                                 /* ENOMEM */
 #include <stdlib.h>                                                /* free() */
 #include <stdio.h>
 #include <string.h>                                            /* strerror() */
-#include <kernel/OS.h>
-#include <View.h>
+#include <InterfaceKit.h>
+#include <DirectWindow.h>
 #include <Application.h>
-#include <Window.h>
-#include <Locker.h>
-#include <Screen.h>
-#include <malloc.h>
-#include <string.h>
+#include <Bitmap.h>
 
 extern "C"
 {
-#include "config.h"
-#include "common.h"
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"
+#include <videolan/vlc.h>
 
 #include "video.h"
 #include "video_output.h"
 
 #include "interface.h"
-#include "intf_msg.h"
 
-#include "main.h"
-
-#include "modules.h"
-#include "modules_export.h"
 }
 
 #include "VideoWindow.h"
-#include <Screen.h>
-
-#define WIDTH 128
-#define HEIGHT 64
-#define BITS_PER_PLANE 16
-#define BYTES_PER_PIXEL 2
+#include "DrawingTidbits.h"
 
 /*****************************************************************************
  * vout_sys_t: BeOS video output method descriptor
@@ -79,21 +56,21 @@ extern "C"
  * This structure is part of the video output thread descriptor.
  * It describes the BeOS specific properties of an output thread.
  *****************************************************************************/
- 
 typedef struct vout_sys_s
 {
-    VideoWindow *         p_window;
+    VideoWindow *  p_window;
+
+    s32 i_width;
+    s32 i_height;
+
+    u8 *pp_buffer[2];
+    int i_index;
 
-    byte_t *              pp_buffer[2];
-    s32                   i_width;
-    s32                   i_height;
 } vout_sys_t;
 
-
 /*****************************************************************************
  * beos_GetAppWindow : retrieve a BWindow pointer from the window name
  *****************************************************************************/
-
 BWindow *beos_GetAppWindow(char *name)
 {
     int32       index;
@@ -116,58 +93,79 @@ BWindow *beos_GetAppWindow(char *name)
     }
     return window; 
 }
-
-/*****************************************************************************
+/****************************************************************************
  * DrawingThread : thread that really does the drawing
- *****************************************************************************/
-
+ ****************************************************************************/
 int32 Draw(void *data)
 {
-  VideoWindow *p_win;
-  p_win = (VideoWindow*) data;
-	  if (p_win->LockLooper())
-      {
-        p_win->view->DrawBitmap( p_win->bitmap[p_win->i_buffer_index],
-                              p_win->bitmap[p_win->i_buffer_index]->Bounds(),
-                              p_win->Bounds());  
-        p_win->UnlockLooper();
-      }
-  return B_OK;
+    VideoWindow* p_win;
+    p_win = (VideoWindow *) data;
+
+    if ( p_win->LockWithTimeout(50000) == B_OK )
+    {
+        if (p_win->vsync)
+        {
+            BScreen *screen;
+            screen = new BScreen(p_win);
+            screen-> WaitForRetrace(22000);
+            delete screen;
+        }
+        if (p_win-> mode == OVERLAY)
+        {
+            p_win-> overlaybitmap->LockBits();
+	        memcpy(p_win-> overlaybitmap->Bits(), 
+	               p_win-> bitmap[p_win-> i_buffer]->Bits(),
+		           p_win-> bitmap[p_win-> i_buffer]->BitsLength() );
+            p_win-> overlaybitmap->UnlockBits();
+            p_win-> view-> Invalidate();
+        }
+        else
+        {
+            p_win-> view-> DrawBitmap( p_win-> bitmap[p_win-> i_buffer], 
+                                       p_win-> view->Bounds() );
+        }                                
+        p_win-> Unlock();
+    }
+    else
+    {
+        intf_Msg("Dropped Frame");
+    }
+    return B_OK;
 }
 
 /*****************************************************************************
  * VideoWindow constructor and destructor
  *****************************************************************************/
-
-VideoWindow::VideoWindow(BRect frame, const char *name, vout_thread_t *p_video_output )
-        : BWindow(frame, name, B_TITLED_WINDOW, B_OUTLINE_RESIZE)
+VideoWindow::VideoWindow( int v_width, int v_height, 
+                          BRect frame )
+            : BWindow( frame, NULL, B_TITLED_WINDOW, 
+                    B_NOT_CLOSABLE | B_NOT_MINIMIZABLE )
 {
+    BView *mainView =  new BView( Bounds(), "mainView", 
+                                  B_FOLLOW_ALL, B_FULL_UPDATE_ON_RESIZE);
+    AddChild(mainView);
+    mainView->SetViewColor(kBlack);
+                                  
+    /* create the view to do the display */
+    view = new VLCView( Bounds() );
+    mainView->AddChild(view);
 
-	/* set the VideoWindow variables */
+    /* set the VideoWindow variables */
     teardownwindow = false;
     is_zoomed = false;
-	p_video_output->b_YCbr = false;
-	
-	/* create the view to do the display */
-    view = new VLCView(Bounds());
-    AddChild(view);
+
+    /* call ScreenChanged to set vsync correctly */
+    BScreen *screen;
+    screen = new BScreen(this);
+    ScreenChanged(screen->Frame(), screen->ColorSpace());
+    delete screen;
     
-    /* Bitmap mode overlay not available */
-	bitmap[0] = new BBitmap(Bounds(), B_RGB32);
-	bitmap[1] = new BBitmap(Bounds(), B_RGB32);
-
-	i_screen_depth = 32;
- 	SetTitle(VOUT_TITLE " (BBitmap output)");
-
-	memset(bitmap[0]->Bits(), 0, bitmap[0]->BitsLength());
- 	memset(bitmap[1]->Bits(), 0, bitmap[1]->BitsLength());
-
-	i_width = bitmap[0]->Bounds().IntegerWidth();
-	i_height = bitmap[0]->Bounds().IntegerHeight();
-	i_bytes_per_pixel = bitmap[0]->BytesPerRow()/i_width;
-    fRowBytes = bitmap[0]->BytesPerRow();
-	i_screen_depth = 8 * i_bytes_per_pixel;
+    // remember current settings
+    i_width = frame.IntegerWidth();
+    i_height = frame.IntegerHeight();
+    FrameResized(frame.IntegerWidth(), frame.IntegerHeight());
 
+    mode = SelectDrawingMode(v_width, v_height);
     Show();
 }
 
@@ -175,113 +173,169 @@ VideoWindow::~VideoWindow()
 {
     int32 result;
 
-    Hide();
-    Sync();
     teardownwindow = true;
+    Lock();
+    view->ClearViewOverlay();
     wait_for_thread(fDrawThreadID, &result);
-   	delete bitmap[0];
-   	delete bitmap[1];
- }
-
-
-/*****************************************************************************
- * VideoWindow::FrameResized
- *****************************************************************************/
-void VideoWindow::FrameResized( float width, float height )
-{
-	if (is_zoomed)
-	{
-	    return;
-	}
-	float width_scale;
-	float height_scale;
-	float orig_width = bitmap[0]->Bounds().Width();
-	float orig_height = bitmap[0]->Bounds().Height();
-
-	width_scale = width / orig_width;
-	height_scale = height / orig_height;
-	
-    /* if the width is proportionally smaller */
-    if (width_scale <= height_scale)
-    {
-        ResizeTo(width, orig_height * width_scale);
-    }
-    else /* if the height is proportionally smaller */
-    {
-        ResizeTo(orig_width * height_scale, height);
-    }
+    delete overlaybitmap;
+    delete bitmap[0];
+    delete bitmap[1];
+    Quit();
 }
 
-/*****************************************************************************
- * VideoWindow::Zoom
- *****************************************************************************/
+void VideoWindow::drawBuffer(int bufferIndex)
+{
+    status_t status;
+
+    i_buffer = bufferIndex;
+    fDrawThreadID = spawn_thread(Draw, "drawing_thread",
+                                 B_DISPLAY_PRIORITY, (void*) this);
+    wait_for_thread(fDrawThreadID, &status);
+}
 
 void VideoWindow::Zoom(BPoint origin, float width, float height )
 {
-	if(is_zoomed)
-	{
-		MoveTo(rect.left, rect.top);
-		ResizeTo(rect.IntegerWidth(), rect.IntegerHeight());
-		be_app->ShowCursor();
-	}
-	else
-	{
-		rect = Frame();
-		BScreen *screen;
-		screen = new BScreen(this);
-		BRect rect = screen->Frame();
-		delete screen;
-		MoveTo(0,0);
-		ResizeTo(rect.IntegerWidth(), rect.IntegerHeight());
-		be_app->HideCursor();
-	}
-	is_zoomed = !is_zoomed;
-}
-
-/*****************************************************************************
- * VideoWindow::MessageReceived
- *****************************************************************************/
-
-void VideoWindow::MessageReceived( BMessage * p_message )
-{
-    BWindow * p_win;
-    
-    switch( p_message->what )
+    if(is_zoomed)
     {
-    case B_KEY_DOWN:
-    case B_SIMPLE_DATA:
-        // post the message to the interface window which will handle it
-        p_win = beos_GetAppWindow( "interface" );
-        if( p_win != NULL )
-        {
-            p_win->PostMessage( p_message );
-        }
-        break;
-    
-    default:
-        BWindow::MessageReceived( p_message );
-        break;
+        is_zoomed = !is_zoomed;
+        MoveTo(winSize.left, winSize.top);
+        ResizeTo(winSize.IntegerWidth(), winSize.IntegerHeight());
+        be_app->ShowCursor();
     }
+    else
+    {
+        is_zoomed = !is_zoomed;
+        BScreen *screen;
+        screen = new BScreen(this);
+        BRect rect = screen->Frame();
+        delete screen;
+        MoveTo(0,0);
+        ResizeTo(rect.IntegerWidth(), rect.IntegerHeight());
+        be_app->ObscureCursor();
+    }
+    view->Invalidate();
 }
 
-/*****************************************************************************
- * VideoWindow::QuitRequested
- *****************************************************************************/
-
-bool VideoWindow::QuitRequested()
+void VideoWindow::FrameMoved(BPoint origin) 
 {
-    /* FIXME: send a message ! */
-    p_main->p_intf->b_die = 1;
-    teardownwindow = true;
-    return( false );
+	if (is_zoomed) return ;
+    winSize = Frame();
+    view->Invalidate();
+}
+
+void VideoWindow::FrameResized( float width, float height )
+{
+    float out_width, out_height;
+    float out_left, out_top;
+    float width_scale = width / i_width;
+    float height_scale = height / i_height;
+
+    if (width_scale <= height_scale)
+    {
+        out_width = (i_width * width_scale);
+        out_height = (i_height * width_scale);
+        out_left = 0; 
+        out_top = (height - out_height) / 2;
+    }
+    else   /* if the height is proportionally smaller */
+    {
+        out_width = (i_width * height_scale);
+        out_height = (i_height * height_scale);
+        out_top = 0;
+        out_left = (width - out_width) /2;
+    }
+    view->MoveTo(out_left,out_top);
+    view->ResizeTo(out_width, out_height);
+	if (!is_zoomed)
+	{
+        winSize = Frame();
+    }
+    view->Invalidate();
+}
+
+void VideoWindow::ScreenChanged(BRect frame, color_space mode)
+{
+    BScreen *screen;
+    float refresh;
+    
+    view->Invalidate();
+    screen = new BScreen(this);
+    display_mode disp_mode; 
+    
+    screen-> GetMode(&disp_mode); 
+    refresh = 
+         (disp_mode.timing.pixel_clock * 1000)/((disp_mode.timing.h_total)* 
+         (disp_mode.timing.v_total)); 
+    if (refresh  < 61) 
+    { 
+        vsync = true; 
+    } 
+}
+
+void VideoWindow::WindowActivated(bool active)
+{
+    view->Invalidate();
+}
+
+int VideoWindow::SelectDrawingMode(int width, int height)
+{
+    int drawingMode = BITMAP;
+
+    int noOverlay = config_GetIntVariable( "nooverlay" );
+    for (int i = 0; i < COLOR_COUNT; i++)
+    {
+        if (noOverlay) break;
+        overlaybitmap = new BBitmap ( BRect( 0, 0, width, height ), 
+                                  B_BITMAP_WILL_OVERLAY|B_BITMAP_RESERVE_OVERLAY_CHANNEL,
+                                  colspace[i].colspace);
+
+        if(overlaybitmap && overlaybitmap->InitCheck() == B_OK) 
+        {
+            overlay_restrictions r;
+            overlaybitmap->GetOverlayRestrictions(&r);
+
+            drawingMode = OVERLAY;
+            colspace_index = i;
+            memset(overlaybitmap->Bits(), 0, overlaybitmap->BitsLength());
+            rgb_color key;
+		    view->SetViewOverlay(overlaybitmap, overlaybitmap->Bounds() ,view->Bounds(),
+		                         &key, B_FOLLOW_ALL,
+			                     B_OVERLAY_FILTER_HORIZONTAL|B_OVERLAY_FILTER_VERTICAL);
+		    view->SetViewColor(key);
+            SetTitle(VOUT_TITLE " (Overlay)");
+            intf_Msg("Color index good: %i", colspace_index);
+            break;
+        }
+        else
+        {
+            delete overlaybitmap;
+            intf_Msg("Color index bad: %i", colspace_index);
+        }        
+	}
+
+    if (drawingMode == BITMAP)
+	{
+        // fallback to RGB32
+        colspace_index = DEFAULT_COL;
+        SetTitle(VOUT_TITLE " (Bitmap)");
+    }
+    
+    bitmap[0] = new BBitmap( BRect( 0, 0, width, height ), colspace[colspace_index].colspace);
+    bitmap[1] = new BBitmap( BRect( 0, 0, width, height ), colspace[colspace_index].colspace);
+    memset(bitmap[0]->Bits(), 0, bitmap[0]->BitsLength());
+    memset(bitmap[1]->Bits(), 0, bitmap[1]->BitsLength());
+    intf_Msg("Color index used: %i", colspace_index);
+    return drawingMode;
 }
 
 /*****************************************************************************
  * VLCView::VLCView
  *****************************************************************************/
-VLCView::VLCView(BRect bounds) : BView(bounds, "", B_FOLLOW_ALL, B_WILL_DRAW)
+VLCView::VLCView(BRect bounds) : BView(bounds, "", B_FOLLOW_NONE,
+                                       B_WILL_DRAW)
+
 {
-	SetViewColor(B_TRANSPARENT_32_BIT);
+    SetViewColor(B_TRANSPARENT_32_BIT);
 }
 
 /*****************************************************************************
@@ -289,35 +343,41 @@ VLCView::VLCView(BRect bounds) : BView(bounds, "", B_FOLLOW_ALL, B_WILL_DRAW)
  *****************************************************************************/
 VLCView::~VLCView()
 {
-
 }
 
 /*****************************************************************************
- * VLCVIew::~VLCView
+ * VLCVIew::MouseDown
  *****************************************************************************/
 void VLCView::MouseDown(BPoint point)
 {
-	VideoWindow *w = (VideoWindow *) Window();
-	if(w->is_zoomed)
-	{
-		BWindow *win = Window();
-		win->Zoom();
-	}
+    BWindow *win = Window();
+    win->Zoom();
 }
 
+/*****************************************************************************
+ * VLCVIew::Draw
+ *****************************************************************************/
+void VLCView::Draw(BRect updateRect) 
+{
+    VideoWindow *win = (VideoWindow *) Window();
+    if (win->mode == BITMAP)
+      FillRect(updateRect);
+}
+
+
 extern "C"
 {
 
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static int  vout_Probe      ( probedata_t *p_data );
-static int  vout_Create     ( struct vout_thread_s * );
-static int  vout_Init       ( struct vout_thread_s * );
-static void vout_End        ( struct vout_thread_s * );
-static void vout_Destroy    ( struct vout_thread_s * );
-static int  vout_Manage     ( struct vout_thread_s * );
-static void vout_Display    ( struct vout_thread_s * );
+static int  vout_Create     ( vout_thread_t * );
+static int  vout_Init       ( vout_thread_t * );
+static void vout_End        ( vout_thread_t * );
+static void vout_Destroy    ( vout_thread_t * );
+static int  vout_Manage     ( vout_thread_t * );
+static void vout_Display    ( vout_thread_t *, picture_t * );
+static void vout_Render     ( vout_thread_t *, picture_t * );
 
 static int  BeosOpenDisplay ( vout_thread_t *p_vout );
 static void BeosCloseDisplay( vout_thread_t *p_vout );
@@ -328,29 +388,13 @@ static void BeosCloseDisplay( vout_thread_t *p_vout );
  *****************************************************************************/
 void _M( vout_getfunctions )( function_list_t * p_function_list )
 {
-    p_function_list->pf_probe = vout_Probe;
     p_function_list->functions.vout.pf_create     = vout_Create;
     p_function_list->functions.vout.pf_init       = vout_Init;
     p_function_list->functions.vout.pf_end        = vout_End;
     p_function_list->functions.vout.pf_destroy    = vout_Destroy;
     p_function_list->functions.vout.pf_manage     = vout_Manage;
     p_function_list->functions.vout.pf_display    = vout_Display;
-    p_function_list->functions.vout.pf_setpalette = NULL;
-}
-
-/*****************************************************************************
- * vout_Probe: probe the video driver and return a score
- *****************************************************************************
- * This function tries to initialize SDL and returns a score to the
- * plugin manager so that it can select the best plugin.
- *****************************************************************************/
-static int vout_Probe( probedata_t *p_data )
-{
-    if( TestMethod( VOUT_METHOD_VAR, "beos" ) )
-    {
-        return( 999 );
-    }
-    return( 100 );
+    p_function_list->functions.vout.pf_render     = vout_Render;
 }
 
 /*****************************************************************************
@@ -367,16 +411,8 @@ int vout_Create( vout_thread_t *p_vout )
         intf_ErrMsg( "error: %s", strerror(ENOMEM) );
         return( 1 );
     }
-    
-	p_vout->b_scale = false;
-
-    /* Open and initialize device */
-    if( BeosOpenDisplay( p_vout ) )
-    {
-        intf_ErrMsg("vout error: can't open display");
-        free( p_vout->p_sys );
-        return( 1 );
-    }
+    p_vout->p_sys->i_width = p_vout->render.i_width;
+    p_vout->p_sys->i_height = p_vout->render.i_height;
 
     return( 0 );
 }
@@ -386,15 +422,67 @@ int vout_Create( vout_thread_t *p_vout )
  *****************************************************************************/
 int vout_Init( vout_thread_t *p_vout )
 {
-    VideoWindow * p_win = p_vout->p_sys->p_window;
-    u32 i_page_size;
-    i_page_size =   p_vout->i_width * p_vout->i_height * p_vout->i_bytes_per_pixel;
-    if((p_win->bitmap[0] != NULL) && (p_win->bitmap[1] != NULL))
-   	{
-    	p_vout->pf_setbuffers( p_vout,
-               (byte_t *)p_win->bitmap[0]->Bits(),
-               (byte_t *)p_win->bitmap[1]->Bits());
+    int i_index;
+    picture_t *p_pic;
+
+    I_OUTPUTPICTURES = 0;
+
+    /* Open and initialize device */
+    if( BeosOpenDisplay( p_vout ) )
+    {
+        intf_ErrMsg("vout error: can't open display");
+        return 0;
     }
+    /* Set the buffers */
+    p_vout->p_sys->pp_buffer[0] = (u8*)p_vout->p_sys->p_window->bitmap[0]->Bits();
+    p_vout->p_sys->pp_buffer[1] = (u8*)p_vout->p_sys->p_window->bitmap[1]->Bits();
+
+    p_vout->output.i_width  = p_vout->render.i_width;
+    p_vout->output.i_height = p_vout->render.i_height;
+
+    /* Assume we have square pixels */
+    p_vout->output.i_aspect = p_vout->p_sys->i_width
+                               * VOUT_ASPECT_FACTOR / p_vout->p_sys->i_height;
+
+    p_vout->output.i_chroma = colspace[p_vout->p_sys->p_window->colspace_index].chroma;
+
+    p_vout->output.i_rmask  = 0x00ff0000;
+    p_vout->output.i_gmask  = 0x0000ff00;
+    p_vout->output.i_bmask  = 0x000000ff;
+
+    p_pic = NULL;
+
+    /* Find an empty picture slot */
+    for( i_index = 0 ; i_index < VOUT_MAX_PICTURES ; i_index++ )
+    {
+        if( p_vout->p_picture[ i_index ].i_status == FREE_PICTURE )
+        {
+            p_pic = p_vout->p_picture + i_index;
+            break;
+        }
+    }
+
+    if( p_pic == NULL )
+    {
+        return 0;
+    }
+
+    p_vout->p_sys->i_index = 0;
+    p_pic->p->p_pixels = p_vout->p_sys->pp_buffer[0];
+    p_pic->p->i_lines = p_vout->p_sys->i_height;
+    p_pic->p->b_margin = 0;
+    p_pic->p->i_pitch = p_vout->p_sys->p_window->bitmap[0]->BytesPerRow();
+    p_pic->p->i_pixel_bytes = p_pic->p->i_pitch / p_vout->p_sys->p_window->bitmap[0]->Bounds().IntegerWidth();
+
+    p_pic->i_planes = colspace[p_vout->p_sys->p_window->colspace_index].planes;
+
+    p_pic->i_status = DESTROYED_PICTURE;
+    p_pic->i_type   = DIRECT_PICTURE;
+
+    PP_OUTPUTPICTURE[ I_OUTPUTPICTURES ] = p_pic;
+
+    I_OUTPUTPICTURES++;
+
     return( 0 );
 }
 
@@ -403,6 +491,7 @@ int vout_Init( vout_thread_t *p_vout )
  *****************************************************************************/
 void vout_End( vout_thread_t *p_vout )
 {
+    BeosCloseDisplay( p_vout );
 }
 
 /*****************************************************************************
@@ -412,7 +501,6 @@ void vout_End( vout_thread_t *p_vout )
  *****************************************************************************/
 void vout_Destroy( vout_thread_t *p_vout )
 {
-    BeosCloseDisplay( p_vout );
     free( p_vout->p_sys );
 }
 
@@ -424,27 +512,16 @@ void vout_Destroy( vout_thread_t *p_vout )
  *****************************************************************************/
 int vout_Manage( vout_thread_t *p_vout )
 {
-    VideoWindow * p_win = p_vout->p_sys->p_window;
-    
-    if ((p_vout->p_buffer[p_vout->i_buffer_index].i_pic_width != p_win->i_width + 1) &&
-        (p_vout->p_buffer[p_vout->i_buffer_index].i_pic_height != p_win->i_height + 1) &&
-        (p_vout->p_buffer[p_vout->i_buffer_index].i_pic_width != 0 ))
-    {
-        if (p_win->Lock())
-        {
-            p_win->view->ClearViewBitmap();
-            intf_Msg("Starting Change");
-            intf_Msg("New width: %d Height: %d", 
-                p_vout->p_buffer[p_vout->i_buffer_index ].i_pic_width,
-                p_vout->p_buffer[p_vout->i_buffer_index ].i_pic_height);
+                          
+    return( 0 );
+}
 
-            p_win->i_width = p_vout->p_buffer[p_vout->i_buffer_index].i_pic_width - 1;
-            p_win->i_height = p_vout->p_buffer[p_vout->i_buffer_index].i_pic_height -1;
-            p_win->ResizeTo((float) p_win->i_width, (float) p_win->i_height); 
-            p_win->Unlock();
-        }
-    }
-	return( 0 );
+/*****************************************************************************
+ * vout_Render: render previously calculated output
+ *****************************************************************************/
+void vout_Render( vout_thread_t *p_vout, picture_t *p_pic )
+{
+    ;
 }
 
 /*****************************************************************************
@@ -453,55 +530,40 @@ int vout_Manage( vout_thread_t *p_vout )
  * This function send the currently rendered image to BeOS image, waits until
  * it is displayed and switch the two rendering buffers, preparing next frame.
  *****************************************************************************/
-void vout_Display( vout_thread_t *p_vout )
+void vout_Display( vout_thread_t *p_vout, picture_t *p_pic )
 {
-	status_t status;
-
     VideoWindow * p_win = p_vout->p_sys->p_window;
-    
-   	p_win->i_buffer_index = p_vout->i_buffer_index;
-	p_vout->i_buffer_index = ++p_vout->i_buffer_index & 1;
-	if (!p_win->teardownwindow)
-	{
-   	   p_win->fDrawThreadID = spawn_thread(Draw, "drawing_thread",
-              B_DISPLAY_PRIORITY, (void*) p_win);
-       wait_for_thread(p_win->fDrawThreadID, &status);
+
+    /* draw buffer if required */    
+    if (!p_win->teardownwindow)
+    { 
+       p_win->drawBuffer(p_vout->p_sys->i_index);
     }
+    /* change buffer */
+    p_vout->p_sys->i_index = ++p_vout->p_sys->i_index & 1;
+    p_pic->p->p_pixels = p_vout->p_sys->pp_buffer[p_vout->p_sys->i_index];
 }
 
 /* following functions are local */
 
 /*****************************************************************************
  * BeosOpenDisplay: open and initialize BeOS device
- *****************************************************************************
- * XXX?? The framebuffer mode is only provided as a fast and efficient way to
- * display video, providing the card is configured and the mode ok. It is
- * not portable, and is not supposed to work with many cards. Use at your
- * own risk !
  *****************************************************************************/
-
 static int BeosOpenDisplay( vout_thread_t *p_vout )
 { 
-    
-    p_vout->p_sys->p_window =
-        new VideoWindow(  BRect( 80, 50, 80+p_vout->i_width-1, 50+p_vout->i_height-1 ), NULL, p_vout );
-    if( p_vout->p_sys->p_window == 0 )
+
+    p_vout->p_sys->p_window = new VideoWindow( p_vout->p_sys->i_width - 1,
+                                               p_vout->p_sys->i_height - 1,
+                                               BRect( 20, 50,
+                                                      20 + p_vout->i_window_width -1, 
+                                                      50 + p_vout->i_window_height ));
+
+    if( p_vout->p_sys->p_window == NULL )
     {
-        free( p_vout->p_sys );
         intf_ErrMsg( "error: cannot allocate memory for VideoWindow" );
         return( 1 );
     }   
-    VideoWindow * p_win = p_vout->p_sys->p_window;
     
-    p_vout->i_screen_depth =         p_win->i_screen_depth;
-    p_vout->i_bytes_per_pixel =      p_win->i_bytes_per_pixel;
-    p_vout->i_width =				 p_win->i_width + 1;
-    p_vout->i_height = 				 p_win->i_height + 1;
-    p_vout->i_bytes_per_line =       p_vout->i_width*p_win->i_bytes_per_pixel;
-
-    p_vout->i_red_mask =        0xff0000;
-    p_vout->i_green_mask =      0x00ff00;
-    p_vout->i_blue_mask =       0x0000ff;
     return( 0 );
 }
 
@@ -514,8 +576,9 @@ static int BeosOpenDisplay( vout_thread_t *p_vout )
 static void BeosCloseDisplay( vout_thread_t *p_vout )
 {    
     /* Destroy the video window */
-    p_vout->p_sys->p_window->Lock();
-    p_vout->p_sys->p_window->Quit();
+    delete p_vout->p_sys->p_window;
 }
 
+
+
 } /* extern "C" */
