@@ -2,7 +2,7 @@
  * ac3_downmix_3dn.c: accelerated 3D Now! ac3 downmix functions
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001 VideoLAN
- * $Id: ac3_downmix_3dn.c,v 1.6.2.1 2001/12/10 01:45:17 sam Exp $
+ * $Id: ac3_downmix_3dn.c,v 1.10 2002/04/01 16:08:23 gbazin Exp $
  *
  * Authors: Renaud Dartus <reno@videolan.org>
  *
@@ -21,21 +21,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME downmix3dn
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
-#include "config.h"
-#include "common.h"
+#include <videolan/vlc.h>
 
 #include "ac3_downmix.h"
 
-/* Defining this one as static will cause dlopen() to segfault */
-const float sqrt2_3dn __asm__ ("sqrt2_3dn") = 0.7071068;
+static const float sqrt2_3dn __asm__ ("sqrt2_3dn") = 0.7071068;
 
 void _M( downmix_3f_2r_to_2ch ) (float * samples, dm_par_t * dm_par)
 {
