@@ -1,10 +1,10 @@
 /*****************************************************************************
  * downmixsse.c : accelerated SSE AC3 downmix module
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: downmixsse.c,v 1.4 2001/07/11 02:01:04 sam Exp $
+ * Copyright (C) 1999-2001 VideoLAN
+ * $Id: downmixsse.c,v 1.8 2002/02/15 13:32:53 sam Exp $
  *
- * Authors: Gal Hendryckx <jimmy@via.ecp.fr>
+ * Authors: Renaud Dartus <reno@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,46 +21,34 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME downmixsse
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <stdlib.h>
+#include <string.h>
 
-#include "config.h"
-#include "common.h"
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"
+#include <videolan/vlc.h>
 
 #include "ac3_downmix.h"
 #include "ac3_downmix_common.h"
 
-#include "modules.h"
-#include "modules_export.h"
-
 /*****************************************************************************
  * Local and extern prototypes.
  *****************************************************************************/
 static void downmix_getfunctions( function_list_t * p_function_list );
-static int  downmix_Probe       ( probedata_t *p_data );
 
 /*****************************************************************************
  * Build configuration tree.
  *****************************************************************************/
 MODULE_CONFIG_START
-ADD_WINDOW( "Configuration for AC3 downmixsse module" )
-    ADD_COMMENT( "Ha, ha -- nothing to configure yet" )
 MODULE_CONFIG_STOP
 
 MODULE_INIT_START
-    p_module->i_capabilities = MODULE_CAPABILITY_NULL
-                                | MODULE_CAPABILITY_DOWNMIX;
-    p_module->psz_longname = "SSE AC3 downmix module";
+    SET_DESCRIPTION( "SSE AC3 downmix module" )
+    ADD_CAPABILITY( DOWNMIX, 200 )
+    ADD_REQUIREMENT( SSE )
+    ADD_SHORTCUT( "sse" )
+    ADD_SHORTCUT( "downmixsse" )
 MODULE_INIT_STOP
 
 MODULE_ACTIVATE_START
@@ -78,7 +66,6 @@ MODULE_DEACTIVATE_STOP
  *****************************************************************************/
 static void downmix_getfunctions( function_list_t * p_function_list )
 {
-    p_function_list->pf_probe = downmix_Probe;
 #define F p_function_list->functions.downmix
     F.pf_downmix_3f_2r_to_2ch = _M( downmix_3f_2r_to_2ch );
     F.pf_downmix_3f_1r_to_2ch = _M( downmix_3f_1r_to_2ch );
@@ -90,23 +77,3 @@ static void downmix_getfunctions( function_list_t * p_function_list )
 #undef F
 }
 
-/*****************************************************************************
- * downmix_Probe: returns a preference score
- *****************************************************************************/
-static int downmix_Probe( probedata_t *p_data )
-{
-    if( !TestCPU( CPU_CAPABILITY_SSE ) )
-    {
-        return( 0 );
-    }
-
-    if( TestMethod( DOWNMIX_METHOD_VAR, "downmixsse" )
-         || TestMethod( DOWNMIX_METHOD_VAR, "sse" ) )
-    {
-        return( 999 );
-    }
-
-    /* This plugin always works */
-    return( 200 );
-}
-
