@@ -1,8 +1,8 @@
 /*****************************************************************************
  * aout_dsp.c : dsp functions library
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: aout_dsp.c,v 1.15 2001/08/24 16:28:49 massiot Exp $
+ * Copyright (C) 1999-2001 VideoLAN
+ * $Id: aout_dsp.c,v 1.26 2002/03/11 07:23:09 gbazin Exp $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -22,9 +22,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME dsp
-#include "modules_inner.h"
-
 /* TODO:
  *
  * - an aout_GetFormats() function
@@ -36,8 +33,6 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <errno.h>                                                 /* ENOMEM */
 #include <fcntl.h>                                       /* open(), O_WRONLY */
 #include <sys/ioctl.h>                                            /* ioctl() */
@@ -46,28 +41,20 @@
 #include <stdio.h>                                           /* "intf_msg.h" */
 #include <stdlib.h>                            /* calloc(), malloc(), free() */
 
-#ifdef SYS_BSD
-#include <machine/soundcard.h>       /* SNDCTL_DSP_RESET, SNDCTL_DSP_SETFMT,
-                   SNDCTL_DSP_STEREO, SNDCTL_DSP_SPEED, SNDCTL_DSP_GETOSPACE */
-#else
-#include <sys/soundcard.h>           /* SNDCTL_DSP_RESET, SNDCTL_DSP_SETFMT,
-                   SNDCTL_DSP_STEREO, SNDCTL_DSP_SPEED, SNDCTL_DSP_GETOSPACE */
+#include <videolan/vlc.h>
+
+/* SNDCTL_DSP_RESET, SNDCTL_DSP_SETFMT, SNDCTL_DSP_STEREO, SNDCTL_DSP_SPEED,
+ * SNDCTL_DSP_GETOSPACE */
+#ifdef HAVE_SOUNDCARD_H
+#   include <soundcard.h>
+#elif defined( HAVE_SYS_SOUNDCARD_H )
+#   include <sys/soundcard.h>
+#elif defined( HAVE_MACHINE_SOUNDCARD_H )
+#   include <machine/soundcard.h>
 #endif
 
-#include "config.h"
-#include "common.h"                                     /* boolean_t, byte_t */
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"
-
 #include "audio_output.h"                                   /* aout_thread_t */
 
-#include "intf_msg.h"                        /* intf_DbgMsg(), intf_ErrMsg() */
-#include "main.h"
-
-#include "modules.h"
-#include "modules_export.h"
-
 /*****************************************************************************
  * aout_sys_t: dsp audio output method descriptor
  *****************************************************************************
@@ -78,15 +65,18 @@ typedef struct aout_sys_s
 {
     audio_buf_info        audio_buf;
 
+    /* Path to the audio output device */
+    char *                psz_device;
+    int                   i_fd;
+
 } aout_sys_t;
 
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
-static int     aout_Probe       ( probedata_t *p_data );
 static int     aout_Open        ( aout_thread_t *p_aout );
 static int     aout_SetFormat   ( aout_thread_t *p_aout );
-static long    aout_GetBufInfo  ( aout_thread_t *p_aout, long l_buffer_info );
+static int     aout_GetBufInfo  ( aout_thread_t *p_aout, int i_buffer_info );
 static void    aout_Play        ( aout_thread_t *p_aout,
                                   byte_t *buffer, int i_size );
 static void    aout_Close       ( aout_thread_t *p_aout );
@@ -97,7 +87,6 @@ static void    aout_Close       ( aout_thread_t *p_aout );
  *****************************************************************************/
 void _M( aout_getfunctions )( function_list_t * p_function_list )
 {
-    p_function_list->pf_probe = aout_Probe;
     p_function_list->functions.aout.pf_open = aout_Open;
     p_function_list->functions.aout.pf_setformat = aout_SetFormat;
     p_function_list->functions.aout.pf_getbufinfo = aout_GetBufInfo;
@@ -105,40 +94,11 @@ void _M( aout_getfunctions )( function_list_t * p_function_list )
     p_function_list->functions.aout.pf_close = aout_Close;
 }
 
-/*****************************************************************************
- * aout_Probe: probe the audio device and return a score
- *****************************************************************************
- * This function tries to open the DSP and returns a score to the plugin
- * manager so that it can choose the most appropriate one.
- *****************************************************************************/
-static int aout_Probe( probedata_t *p_data )
-{
-    char * psz_device = main_GetPszVariable( AOUT_DSP_VAR, AOUT_DSP_DEFAULT );
-    int i_fd = open( psz_device, O_WRONLY|O_NONBLOCK );
-
-    /* If we were unable to open the device, there is no way we can use
-     * the plugin. Return a score of 0. */
-    if( i_fd < 0 )
-    {
-        return( 0 );
-    }
-
-    /* Otherwise, there are good chances we can use this plugin, return 100. */
-    close( i_fd );
-
-    if( TestMethod( AOUT_METHOD_VAR, "dsp" ) )
-    {
-        return( 999 );
-    }
-
-    return( 100 );
-}
-
 /*****************************************************************************
  * aout_Open: opens the audio device (the digital sound processor)
  *****************************************************************************
  * This function opens the dsp as a usual non-blocking write-only file, and
- * modifies the p_aout->i_fd with the file's descriptor.
+ * modifies the p_aout->p_sys->i_fd with the file's descriptor.
  *****************************************************************************/
 static int aout_Open( aout_thread_t *p_aout )
 {
@@ -151,18 +111,21 @@ static int aout_Open( aout_thread_t *p_aout )
     }
 
     /* Initialize some variables */
-    p_aout->psz_device = main_GetPszVariable( AOUT_DSP_VAR, AOUT_DSP_DEFAULT );
-    p_aout->i_format   = AOUT_FORMAT_DEFAULT;
-    p_aout->i_channels = 1 + main_GetIntVariable( AOUT_STEREO_VAR,
-                                                  AOUT_STEREO_DEFAULT );
-    p_aout->l_rate     = main_GetIntVariable( AOUT_RATE_VAR,
-                                              AOUT_RATE_DEFAULT );
+    if( !(p_aout->p_sys->psz_device = config_GetPszVariable( "dsp_dev" )) )
+    {
+        intf_ErrMsg( "aout error: don't know which audio device to open" );
+        free( p_aout->p_sys );
+        return( -1 );
+    }
 
     /* Open the sound device */
-    if( (p_aout->i_fd = open( p_aout->psz_device, O_WRONLY )) < 0 )
+    if( (p_aout->p_sys->i_fd = open( p_aout->p_sys->psz_device, O_WRONLY ))
+        < 0 )
     {
         intf_ErrMsg( "aout error: can't open audio device (%s)",
-                     p_aout->psz_device );
+                     p_aout->p_sys->psz_device );
+        free( p_aout->p_sys->psz_device );
+        free( p_aout->p_sys );
         return( -1 );
     }
 
@@ -180,20 +143,20 @@ static int aout_Open( aout_thread_t *p_aout )
 static int aout_SetFormat( aout_thread_t *p_aout )
 {
     int i_format;
-    long l_rate;
-    boolean_t b_stereo = p_aout->b_stereo;
+    int i_rate;
+    boolean_t b_stereo;
 
     /* Reset the DSP device */
-    if( ioctl( p_aout->i_fd, SNDCTL_DSP_RESET, NULL ) < 0 )
+    if( ioctl( p_aout->p_sys->i_fd, SNDCTL_DSP_RESET, NULL ) < 0 )
     {
         intf_ErrMsg( "aout error: can't reset audio device (%s)",
-                     p_aout->psz_device );
+                     p_aout->p_sys->psz_device );
         return( -1 );
     }
 
     /* Set the output format */
     i_format = p_aout->i_format;
-    if( ioctl( p_aout->i_fd, SNDCTL_DSP_SETFMT, &i_format ) < 0 )
+    if( ioctl( p_aout->p_sys->i_fd, SNDCTL_DSP_SETFMT, &i_format ) < 0 )
     {
         intf_ErrMsg( "aout error: can't set audio output format (%i)",
                      p_aout->i_format );
@@ -202,45 +165,44 @@ static int aout_SetFormat( aout_thread_t *p_aout )
 
     if( i_format != p_aout->i_format )
     {
-        intf_WarnMsg( 2, "aout warning: audio output format not supported (%i)",
-                      p_aout->i_format );
+        intf_WarnMsg( 2, "aout warning: audio output format not supported (%i)"
+                      ,p_aout->i_format );
         p_aout->i_format = i_format;
     }
 
     /* Set the number of channels */
-    if( ioctl( p_aout->i_fd, SNDCTL_DSP_STEREO, &b_stereo ) < 0 )
+    b_stereo = ( p_aout->i_channels >= 2 );
+
+    if( ioctl( p_aout->p_sys->i_fd, SNDCTL_DSP_STEREO, &b_stereo ) < 0 )
     {
         intf_ErrMsg( "aout error: can't set number of audio channels (%i)",
                      p_aout->i_channels );
         return( -1 );
     }
 
-    if( b_stereo != p_aout->b_stereo )
+    if( (1 + b_stereo) != p_aout->i_channels )
     {
-        intf_WarnMsg( 2, "aout warning: number of audio channels not supported (%i)",
+        intf_WarnMsg( 2, "aout warning: %i audio channels not supported",
                       p_aout->i_channels );
-        p_aout->b_stereo = b_stereo;
         p_aout->i_channels = 1 + b_stereo;
     }
 
     /* Set the output rate */
-    l_rate = p_aout->l_rate;
-    if( ioctl( p_aout->i_fd, SNDCTL_DSP_SPEED, &l_rate ) < 0 )
+    i_rate = p_aout->i_rate;
+    if( ioctl( p_aout->p_sys->i_fd, SNDCTL_DSP_SPEED, &i_rate ) < 0 )
     {
-        intf_ErrMsg( "aout error: can't set audio output rate (%li)",
-                     p_aout->l_rate );
+        intf_ErrMsg( "aout error: can't set audio output rate (%i)",
+                     p_aout->i_rate );
         return( -1 );
     }
 
-    if( l_rate != p_aout->l_rate )
+    if( i_rate != p_aout->i_rate )
     {
         intf_WarnMsg( 1, "aout warning: audio output rate not supported (%li)",
-                      p_aout->l_rate );
-        p_aout->l_rate = l_rate;
+                      p_aout->i_rate );
+        p_aout->i_rate = i_rate;
     }
 
-    p_aout->i_latency = 0;
-    
     return( 0 );
 }
 
@@ -248,16 +210,16 @@ static int aout_SetFormat( aout_thread_t *p_aout )
  * aout_GetBufInfo: buffer status query
  *****************************************************************************
  * This function fills in the audio_buf_info structure :
- * - int fragments : number of available fragments (partially usend ones not
- *   counted)
+ * - returns : number of available fragments (not partially used ones)
  * - int fragstotal : total number of fragments allocated
  * - int fragsize : size of a fragment in bytes
  * - int bytes : available space in bytes (includes partially used fragments)
  * Note! 'bytes' could be more than fragments*fragsize
  *****************************************************************************/
-static long aout_GetBufInfo( aout_thread_t *p_aout, long l_buffer_limit )
+static int aout_GetBufInfo( aout_thread_t *p_aout, int i_buffer_limit )
 {
-    ioctl( p_aout->i_fd, SNDCTL_DSP_GETOSPACE, &p_aout->p_sys->audio_buf );
+    ioctl( p_aout->p_sys->i_fd, SNDCTL_DSP_GETOSPACE,
+           &p_aout->p_sys->audio_buf );
 
     /* returns the allocated space in bytes */
     return ( (p_aout->p_sys->audio_buf.fragstotal
@@ -274,7 +236,11 @@ static void aout_Play( aout_thread_t *p_aout, byte_t *buffer, int i_size )
 {
     if( p_aout->b_active )
     {
-        write( p_aout->i_fd, buffer, i_size );
+        int i_tmp;
+        i_tmp = write( p_aout->p_sys->i_fd, buffer, i_size );
+	if( i_tmp < 0 )
+	    intf_ErrMsg("aout error: %s", strerror(ENOMEM) );
+
     }
 }
 
@@ -283,6 +249,6 @@ static void aout_Play( aout_thread_t *p_aout, byte_t *buffer, int i_size )
  *****************************************************************************/
 static void aout_Close( aout_thread_t *p_aout )
 {
-    close( p_aout->i_fd );
+    close( p_aout->p_sys->i_fd );
+    free( p_aout->p_sys->psz_device );
 }
-
