@@ -2,7 +2,7 @@
  * dummy.c : dummy plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000, 2001 VideoLAN
- * $Id: dummy.c,v 1.11 2001/11/28 15:08:05 massiot Exp $
+ * $Id: dummy.c,v 1.17 2002/03/01 00:33:18 massiot Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -21,30 +21,19 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME dummy
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <stdlib.h>                                      /* malloc(), free() */
 #include <string.h>
 
-#include "config.h"
-#include "common.h"                                     /* boolean_t, byte_t */
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
-
-#include "modules.h"
-#include "modules_export.h"
+#include <videolan/vlc.h>
 
 /*****************************************************************************
  * Capabilities defined in the other files.
  *****************************************************************************/
-void _M( input_getfunctions ) ( function_list_t * p_function_list );
+void _M( access_getfunctions ) ( function_list_t * p_function_list );
+void _M( demux_getfunctions ) ( function_list_t * p_function_list );
 void _M( aout_getfunctions )  ( function_list_t * p_function_list );
 void _M( vout_getfunctions )  ( function_list_t * p_function_list );
 void _M( intf_getfunctions )  ( function_list_t * p_function_list );
@@ -53,26 +42,32 @@ void _M( intf_getfunctions )  ( function_list_t * p_function_list );
  * Build configuration tree.
  *****************************************************************************/
 MODULE_CONFIG_START
-ADD_WINDOW( "Configuration for dummy module" )
-    ADD_COMMENT( "Ha, ha -- nothing to configure yet" )
 MODULE_CONFIG_STOP
 
+
 MODULE_INIT_START
-    p_module->i_capabilities = MODULE_CAPABILITY_NULL
-                                | MODULE_CAPABILITY_INPUT
-                                | MODULE_CAPABILITY_AOUT
-                                | MODULE_CAPABILITY_VOUT
-                                | MODULE_CAPABILITY_INTF;
-    p_module->psz_longname = "dummy functions module";
+    SET_DESCRIPTION( "dummy functions module" )
+    /* Capability score set to 0 because we don't want to be spawned
+     * unless explicitly requested to */
+    ADD_CAPABILITY( AOUT, 0 )
+    ADD_CAPABILITY( VOUT, 0 )
+    ADD_CAPABILITY( INTF, 0 )
+    ADD_CAPABILITY( ACCESS, 0 )
+    ADD_CAPABILITY( DEMUX, 0 )
+    ADD_SHORTCUT( "dummy" )
+    ADD_SHORTCUT( "vlc" )
 MODULE_INIT_STOP
 
+
 MODULE_ACTIVATE_START
-    _M( input_getfunctions )( &p_module->p_functions->input );
+    _M( access_getfunctions )( &p_module->p_functions->access );
+    _M( demux_getfunctions )( &p_module->p_functions->demux );
     _M( aout_getfunctions )( &p_module->p_functions->aout );
     _M( vout_getfunctions )( &p_module->p_functions->vout );
     _M( intf_getfunctions )( &p_module->p_functions->intf );
 MODULE_ACTIVATE_STOP
 
+
 MODULE_DEACTIVATE_START
 MODULE_DEACTIVATE_STOP
 
