@@ -2,7 +2,7 @@
  * null.c : NULL module for vlc
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: null.c,v 1.2 2001/11/28 15:08:05 massiot Exp $
+ * $Id: null.c,v 1.5 2002/02/24 20:51:09 gbazin Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -21,55 +21,45 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME null
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <stdlib.h>                                      /* malloc(), free() */
 #include <string.h>                                              /* strdup() */
 
-#include "config.h"
-#include "common.h"                                     /* boolean_t, byte_t */
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
-
-#include "modules.h"
-#include "modules_inner.h"
-#include "modules_export.h"
+#include <videolan/vlc.h>
 
 /*****************************************************************************
  * Build configuration tree.
  *****************************************************************************/
 MODULE_CONFIG_START
-ADD_WINDOW( "Configuration for null module" )
-    ADD_PANE( "First pane" )
-        ADD_FRAME( "First frame" )
-            ADD_COMMENT( "You can put whatever you want here." )
-            ADD_STRING( "Random text: ", MODULE_VAR(text), NULL )
-        ADD_FRAME( "Second frame" )
-            ADD_COMMENT( "The file below is not used." )
-            ADD_FILE( "Select file: ", MODULE_VAR(file), NULL )
-        ADD_FRAME( "Third frame" )
-            ADD_COMMENT( "This space intentionally left blank." )
-    ADD_PANE( "Second pane" )
-        ADD_FRAME( "Frame" )
-            ADD_COMMENT( "There is nothing in this frame." )
+     /*        ADD_PANE( "First pane" )
+            ADD_SUBCATEGORY_HINT( "First frame" )
+
+                ADD_STRING( "Random text: ", MODULE_VAR(text), NULL )
+            ADD_SUBCATEGORY_HINT( "Second frame" )
+
+                ADD_FILE( "Select file: ", MODULE_VAR(file), NULL )
+            ADD_SUBCATEGORY_HINT( "Third frame" )
+
+        ADD_PANE( "Second pane" )
+            ADD_SUBCATEGORY_HINT( "Frame" )
+     */
 MODULE_CONFIG_STOP
 
+
 MODULE_INIT_START
-    p_module->i_capabilities = MODULE_CAPABILITY_NULL;
-    p_module->psz_longname = "the Null module that does nothing";
+    SET_DESCRIPTION( "the Null module that does nothing" )
+    ADD_SHORTCUT( "null" )
 MODULE_INIT_STOP
 
+
 MODULE_ACTIVATE_START
     /* Since the Null module can't do anything, there is no need to
      * fill the p_functions structure. */
 MODULE_ACTIVATE_STOP
 
+
 MODULE_DEACTIVATE_START
 MODULE_DEACTIVATE_STOP
 
