@@ -2,7 +2,7 @@
  * gtk_open.c : functions to handle file/disc/network open widgets.
  *****************************************************************************
  * Copyright (C) 2000, 2001 VideoLAN
- * $Id: gtk_open.c,v 1.11 2001/12/07 18:33:07 sam Exp $
+ * $Id: gtk_open.c,v 1.20 2002/03/25 02:06:24 jobi Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Stphane Borel <stef@via.ecp.fr>
@@ -25,27 +25,19 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
 #include <sys/types.h>                                              /* off_t */
 #include <stdlib.h>
 
-#define gtk 12
-#define gnome 42
-#if ( MODULE_NAME == gtk )
-#   include <gtk/gtk.h>
-#elif ( MODULE_NAME == gnome )
+#include <videolan/vlc.h>
+
+#ifdef MODULE_NAME_IS_gnome
 #   include <gnome.h>
+#else
+#   include <gtk/gtk.h>
 #endif
-#undef gtk
-#undef gnome
 
 #include <string.h>
 
-#include "common.h"
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
-
 #include "stream_control.h"
 #include "input_ext-intf.h"
 
@@ -56,12 +48,10 @@
 #include "gtk_interface.h"
 #include "gtk_support.h"
 #include "gtk_playlist.h"
-#include "intf_gtk.h"
+#include "gtk_common.h"
 
 #include "netutils.h"
 
-#include "modules_export.h"
-
 /*****************************************************************************
  * Fileopen callbacks
  *****************************************************************************
@@ -76,13 +66,16 @@ gboolean GtkFileOpenShow( GtkWidget       *widget,
     /* If we have never used the file selector, open it */
     if( !GTK_IS_WIDGET( p_intf->p_sys->p_fileopen ) )
     {
+        char *psz_path;
+
         p_intf->p_sys->p_fileopen = create_intf_fileopen();
         gtk_object_set_data( GTK_OBJECT( p_intf->p_sys->p_fileopen ),
                              "p_intf", p_intf );
 
-        gtk_file_selection_set_filename( GTK_FILE_SELECTION(
-            p_intf->p_sys->p_fileopen ),
-            main_GetPszVariable( INTF_PATH_VAR, INTF_PATH_DEFAULT ) );
+        if( (psz_path = config_GetPszVariable( "search_path" )) )
+            gtk_file_selection_set_filename( GTK_FILE_SELECTION(
+                p_intf->p_sys->p_fileopen ), psz_path );
+        if( psz_path ) free( psz_path );
     }
 
     gtk_widget_show( p_intf->p_sys->p_fileopen );
@@ -121,9 +114,9 @@ void GtkFileOpenOk( GtkButton * button, gpointer user_data )
     GtkRebuildCList( p_playlist_clist, p_main->p_playlist );
 
     /* end current item, select added item  */
-    if( p_intf->p_input != NULL )
+    if( p_input_bank->pp_input[0] != NULL )
     {
-        p_intf->p_input->b_eof = 1;
+        p_input_bank->pp_input[0]->b_eof = 1;
     }
 
     intf_PlaylistJumpto( p_main->p_playlist, i_end - 1 );
@@ -158,10 +151,13 @@ void GtkDiscOpenDvd( GtkToggleButton * togglebutton, gpointer user_data )
 {
     if( togglebutton->active )
     {
-        gtk_entry_set_text(
-          GTK_ENTRY( lookup_widget( GTK_WIDGET(togglebutton), "disc_name" ) ),
-          main_GetPszVariable( INPUT_DVD_DEVICE_VAR, INPUT_DVD_DEVICE_DEFAULT )
-        );
+        char *psz_dvd_device;
+
+        if( (psz_dvd_device = config_GetPszVariable( "dvd_device" )) )
+            gtk_entry_set_text(
+                GTK_ENTRY( lookup_widget( GTK_WIDGET(togglebutton),
+                                          "disc_name" ) ), psz_dvd_device );
+        if( psz_dvd_device ) free( psz_dvd_device );
     }
 }
 
@@ -169,10 +165,13 @@ void GtkDiscOpenVcd( GtkToggleButton * togglebutton, gpointer user_data )
 {
     if( togglebutton->active )
     {
-        gtk_entry_set_text(
-          GTK_ENTRY( lookup_widget( GTK_WIDGET(togglebutton), "disc_name" ) ),
-          main_GetPszVariable( INPUT_VCD_DEVICE_VAR, INPUT_VCD_DEVICE_DEFAULT )
-        );
+        char *psz_vcd_device;
+
+        if( (psz_vcd_device = config_GetPszVariable( "vcd_device" )) )
+            gtk_entry_set_text(
+                GTK_ENTRY( lookup_widget( GTK_WIDGET(togglebutton),
+                                          "disc_name" ) ), psz_vcd_device );
+        if( psz_vcd_device ) free( psz_vcd_device );
     }
 }
 
@@ -182,19 +181,12 @@ void GtkDiscOpenOk( GtkButton * button, gpointer user_data )
     GtkCList *      p_playlist_clist;
     char *          psz_device, *psz_source, *psz_method;
     int             i_end = p_main->p_playlist->i_size;
+    int             i_title, i_chapter;
 
     gtk_widget_hide( p_intf->p_sys->p_disc );
     psz_device = gtk_entry_get_text( GTK_ENTRY( lookup_widget(
                                          GTK_WIDGET(button), "disc_name" ) ) );
 
-    /* "dvd:foo" has size 5 + strlen(foo) */
-    psz_source = malloc( 3 /* "dvd" */ + 1 /* ":" */
-                           + strlen( psz_device ) + 1 /* "\0" */ );
-    if( psz_source == NULL )
-    {
-        return;
-    }
-
     /* Check which method was activated */
     if( GTK_TOGGLE_BUTTON( lookup_widget( GTK_WIDGET(button),
                                           "disc_dvd" ) )->active )
@@ -209,21 +201,30 @@ void GtkDiscOpenOk( GtkButton * button, gpointer user_data )
     else
     {
         intf_ErrMsg( "intf error: unknown disc type toggle button position" );
-        free( psz_source );
         return;
     }
     
     /* Select title and chapter */
-    main_PutIntVariable( INPUT_TITLE_VAR, gtk_spin_button_get_value_as_int(
+    i_title = gtk_spin_button_get_value_as_int(
                               GTK_SPIN_BUTTON( lookup_widget(
-                                  GTK_WIDGET(button), "disc_title" ) ) ) );
+                                  GTK_WIDGET(button), "disc_title" ) ) );
 
-    main_PutIntVariable( INPUT_CHAPTER_VAR, gtk_spin_button_get_value_as_int(
+    i_chapter = gtk_spin_button_get_value_as_int(
                               GTK_SPIN_BUTTON( lookup_widget(
-                                  GTK_WIDGET(button), "disc_chapter" ) ) ) );
+                                  GTK_WIDGET(button), "disc_chapter" ) ) );
+    
+    /* "dvd:foo" has size 5 + strlen(foo) */
+    psz_source = malloc( 3 /* "dvd" */ + 1 /* ":" */
+                           + strlen( psz_device ) + 2 /* @, */
+                           + 4 /* i_title & i_chapter < 100 */ + 1 /* "\0" */ );
+    if( psz_source == NULL )
+    {
+        return;
+    }
 
     /* Build source name and add it to playlist */
-    sprintf( psz_source, "%s:%s", psz_method, psz_device );
+    sprintf( psz_source, "%s:%s@%d,%d",
+             psz_method, psz_device, i_title, i_chapter );
     intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END, psz_source );
     free( psz_source );
 
@@ -235,9 +236,9 @@ void GtkDiscOpenOk( GtkButton * button, gpointer user_data )
     GtkRebuildCList( p_playlist_clist, p_main->p_playlist );
 
     /* stop current item, select added item */
-    if( p_intf->p_input != NULL )
+    if( p_input_bank->pp_input[0] != NULL )
     {
-        p_intf->p_input->b_eof = 1;
+        p_input_bank->pp_input[0]->b_eof = 1;
     }
 
     intf_PlaylistJumpto( p_main->p_playlist, i_end - 1 );
@@ -263,44 +264,30 @@ gboolean GtkNetworkOpenShow( GtkWidget       *widget,
 
     if( !GTK_IS_WIDGET( p_intf->p_sys->p_network ) )
     {
+        char *psz_channel_server;
+
         p_intf->p_sys->p_network = create_intf_network();
         gtk_object_set_data( GTK_OBJECT( p_intf->p_sys->p_network ),
                              "p_intf", p_intf );
 
-        gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data(
-            GTK_OBJECT( p_intf->p_sys->p_network ), "network_server" ) ),
-            main_GetPszVariable( INPUT_SERVER_VAR,
-                                 INPUT_SERVER_DEFAULT ) );
-
         gtk_spin_button_set_value( GTK_SPIN_BUTTON( gtk_object_get_data(
             GTK_OBJECT( p_intf->p_sys->p_network ), "network_port" ) ),
-            main_GetIntVariable( INPUT_PORT_VAR,
-                                 INPUT_PORT_DEFAULT ) );
+            config_GetIntVariable( "server_port" ) );
 
-        gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data(
-            GTK_OBJECT( p_intf->p_sys->p_network ), "network_broadcast" ) ),
-            main_GetPszVariable( INPUT_BCAST_ADDR_VAR,
-                                 INPUT_BCAST_ADDR_DEFAULT ) );
-
-        gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data(
-            GTK_OBJECT( p_intf->p_sys->p_network ), "network_channel" ) ),
-            main_GetPszVariable( INPUT_CHANNEL_SERVER_VAR,
-                                 INPUT_CHANNEL_SERVER_DEFAULT ) );
+        psz_channel_server = config_GetPszVariable( "channel_server" );
+        if( psz_channel_server )
+            gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data(
+                GTK_OBJECT( p_intf->p_sys->p_network ), "network_channel" ) ),
+                psz_channel_server );
+        if( psz_channel_server ) free( psz_channel_server );
 
         gtk_spin_button_set_value( GTK_SPIN_BUTTON( gtk_object_get_data(
             GTK_OBJECT( p_intf->p_sys->p_network ), "network_channel_port" ) ),
-            main_GetIntVariable( INPUT_CHANNEL_PORT_VAR,
-                                 INPUT_CHANNEL_PORT_DEFAULT ) );
+            config_GetIntVariable( "channel_port" ) );
 
         gtk_toggle_button_set_active( gtk_object_get_data( GTK_OBJECT(
             p_intf->p_sys->p_network ), "network_channel_check" ),
-            main_GetIntVariable( INPUT_NETWORK_CHANNEL_VAR,
-                                 INPUT_NETWORK_CHANNEL_DEFAULT ) );
-            
-        gtk_toggle_button_set_active( gtk_object_get_data( GTK_OBJECT(
-            p_intf->p_sys->p_network ), "network_broadcast_check" ),
-            main_GetIntVariable( INPUT_BROADCAST_VAR,
-                                 INPUT_BROADCAST_DEFAULT ) );
+            config_GetIntVariable( "network_channel" ) );
     }
 
     gtk_widget_show( p_intf->p_sys->p_network );
@@ -325,9 +312,9 @@ void GtkNetworkOpenOk( GtkButton *button, gpointer user_data )
                                  GTK_WIDGET(button), "network_server" ) ) );
 
     /* select added item */
-    if( p_intf->p_input != NULL )
+    if( p_input_bank->pp_input[0] != NULL )
     {
-        p_intf->p_input->b_eof = 1;
+        p_input_bank->pp_input[0]->b_eof = 1;
     }
 
     /* Check which protocol was activated */
@@ -350,7 +337,7 @@ void GtkNetworkOpenOk( GtkButton *button, gpointer user_data )
     /* Manage channel server */
     b_channel = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(
         lookup_widget( GTK_WIDGET(button), "network_channel_check" ) ) );
-    main_PutIntVariable( INPUT_NETWORK_CHANNEL_VAR, b_channel );
+    config_PutIntVariable( "network_channel", b_channel );
     if( b_channel )
     {
         char *          psz_channel;
@@ -366,10 +353,10 @@ void GtkNetworkOpenOk( GtkButton *button, gpointer user_data )
         i_channel_port = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(
             lookup_widget( GTK_WIDGET(button), "network_channel_port" ) ) );
 
-        main_PutPszVariable( INPUT_CHANNEL_SERVER_VAR, psz_channel );
+        config_PutPszVariable( "channel_server", psz_channel );
         if( i_channel_port < 65536 )
         {
-            main_PutIntVariable( INPUT_CHANNEL_PORT_VAR, i_channel_port );
+            config_PutIntVariable( "channel_port", i_channel_port );
         }
 
         p_intf->p_sys->b_playing = 1;
@@ -396,7 +383,7 @@ void GtkNetworkOpenOk( GtkButton *button, gpointer user_data )
                             GTK_WIDGET(button), "network_broadcast" ) ) );
             /* Allocate room for "protocol://server:port" */
             psz_source = malloc( strlen( psz_protocol ) + 3 /* "://" */
-                                   + strlen( psz_server ) + 1 /* ":" */
+                                   + strlen( psz_server ) + 2 /* "@:" */
                                    + 5 /* 0-65535 */
                                    + strlen( psz_broadcast ) + 2 /* "::" */ 
                                    + 1 /* "\0" */ );
@@ -406,7 +393,7 @@ void GtkNetworkOpenOk( GtkButton *button, gpointer user_data )
             }
 
             /* Build source name and add it to playlist */
-            sprintf( psz_source, "%s://%s:%i/%s", psz_protocol,
+            sprintf( psz_source, "%s://%s@:%i/%s", psz_protocol,
                                                   psz_server,
                                                   i_port,
                                                   psz_broadcast );
@@ -415,7 +402,7 @@ void GtkNetworkOpenOk( GtkButton *button, gpointer user_data )
         {
             /* Allocate room for "protocol://server:port" */
             psz_source = malloc( strlen( psz_protocol ) + 3 /* "://" */
-                                   + strlen( psz_server ) + 1 /* ":" */
+                                   + strlen( psz_server ) + 2 /* "@:" */
                                    + 5 /* 0-65535 */ + 1 /* "\0" */ );
             if( psz_source == NULL )
             {
@@ -423,7 +410,7 @@ void GtkNetworkOpenOk( GtkButton *button, gpointer user_data )
             }
            
             /* Build source name and add it to playlist */
-            sprintf( psz_source, "%s://%s:%i",
+            sprintf( psz_source, "%s://%s@:%i",
                      psz_protocol, psz_server, i_port );
         }
 
@@ -512,6 +499,94 @@ void GtkNetworkOpenChannel( GtkToggleButton * togglebutton,
             "network_broadcast" ), b_broadcast && ! b_channel );
 }
 
+/*****************************************************************************
+ * Open satellite callbacks
+ *****************************************************************************
+ * The following callbacks are related to the satellite card manager.
+ *****************************************************************************/
+gboolean GtkSatOpenShow( GtkWidget       *widget,
+                          GdkEventButton  *event,
+                          gpointer         user_data)
+{
+    intf_thread_t *p_intf = GetIntf( GTK_WIDGET(widget), (char*)user_data );
+
+    if( !GTK_IS_WIDGET( p_intf->p_sys->p_sat ) )
+    {
+        p_intf->p_sys->p_sat = create_intf_sat();
+        gtk_object_set_data( GTK_OBJECT( p_intf->p_sys->p_sat ),
+                             "p_intf", p_intf );
+    }
+
+    gtk_widget_show( p_intf->p_sys->p_sat );
+    gdk_window_raise( p_intf->p_sys->p_sat->window );
+
+    return TRUE;
+}
+
+void GtkSatOpenOk( GtkButton * button, gpointer user_data )
+{
+    intf_thread_t * p_intf = GetIntf( GTK_WIDGET(button), "intf_sat" );
+    GtkCList *      p_playlist_clist;
+    char *          psz_source;
+    int             i_end = p_main->p_playlist->i_size;
+    int             i_freq, i_srate;
+    boolean_t       b_pol;
+
+    gtk_widget_hide( p_intf->p_sys->p_sat );
+
+    /* Check which polarization was activated */
+    if( GTK_TOGGLE_BUTTON( lookup_widget( GTK_WIDGET(button),
+                                          "sat_pol_vert" ) )->active )
+    {
+        b_pol = 0;
+    }
+    else
+    {
+        b_pol = 1;
+    }
+    
+    /* Select frequency and symbol rate */
+    i_freq = gtk_spin_button_get_value_as_int(
+                              GTK_SPIN_BUTTON( lookup_widget(
+                                  GTK_WIDGET(button), "sat_freq" ) ) );
+
+    i_srate = gtk_spin_button_get_value_as_int(
+                              GTK_SPIN_BUTTON( lookup_widget(
+                                  GTK_WIDGET(button), "sat_srate" ) ) );
+    
+    psz_source = malloc( 22 );
+    if( psz_source == NULL )
+    {
+        return;
+    }
+
+    /* Build source name and add it to playlist */
+    sprintf( psz_source, "%s:%d,%d,%d",
+             "satellite", i_freq, b_pol, i_srate );
+    intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END, psz_source );
+    free( psz_source );
+
+    /* catch the GTK CList */
+    p_playlist_clist = GTK_CLIST( gtk_object_get_data(
+        GTK_OBJECT( p_intf->p_sys->p_playlist ), "playlist_clist" ) );
+
+    /* update the display */
+    GtkRebuildCList( p_playlist_clist, p_main->p_playlist );
+
+    /* stop current item, select added item */
+    if( p_input_bank->pp_input[0] != NULL )
+    {
+        p_input_bank->pp_input[0]->b_eof = 1;
+    }
+
+    intf_PlaylistJumpto( p_main->p_playlist, i_end - 1 );
+}
+
+
+void GtkSatOpenCancel( GtkButton * button, gpointer user_data )
+{
+    gtk_widget_hide( gtk_widget_get_toplevel( GTK_WIDGET (button) ) );
+}
 
 /****************************************************************************
  * Callbacks for menuitem
