@@ -2,7 +2,7 @@
  * idctaltivec.c : Altivec IDCT module
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: idctaltivec.c,v 1.20 2001/12/06 13:46:23 massiot Exp $
+ * $Id: idctaltivec.c,v 1.23 2002/02/15 13:32:53 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -23,30 +23,17 @@
 
 #ifndef __BUILD_ALTIVEC_ASM__
 
-#define MODULE_NAME idctaltivec
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <stdlib.h>
 #include <string.h>
 #include <inttypes.h>
 
-#include "config.h"
-#include "common.h"
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"                                              /* TestCPU() */
+#include <videolan/vlc.h>
 
 #include "idct.h"
 
-#include "modules.h"
-#include "modules_export.h"
-
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
@@ -56,14 +43,14 @@ static void idct_getfunctions( function_list_t * p_function_list );
  * Build configuration tree.
  *****************************************************************************/
 MODULE_CONFIG_START
-ADD_WINDOW( "Configuration for Altivec IDCT module" )
-    ADD_COMMENT( "Ha, ha -- nothing to configure yet" )
 MODULE_CONFIG_STOP
 
 MODULE_INIT_START
-    p_module->i_capabilities = MODULE_CAPABILITY_NULL
-                                | MODULE_CAPABILITY_IDCT;
-    p_module->psz_longname = "Altivec IDCT module";
+    SET_DESCRIPTION( "Altivec IDCT module" )
+    ADD_CAPABILITY( IDCT, 200 )
+    ADD_REQUIREMENT( ALTIVEC )
+    ADD_SHORTCUT( "altivec" )
+    ADD_SHORTCUT( "idctaltivec" )
 MODULE_INIT_STOP
 
 MODULE_ACTIVATE_START
@@ -75,25 +62,6 @@ MODULE_DEACTIVATE_STOP
 
 /* Following functions are local */
 
-/*****************************************************************************
- * idct_Probe: return a preference score
- *****************************************************************************/
-static int idct_Probe( probedata_t *p_data )
-{
-    if( !TestCPU( CPU_CAPABILITY_ALTIVEC ) )
-    {
-        return( 0 );
-    }
-
-    if( TestMethod( IDCT_METHOD_VAR, "idctaltivec" )
-         || TestMethod( IDCT_METHOD_VAR, "altivec" ) )
-    {
-        return( 999 );
-    }
-
-    return( 200 );
-}
-
 /*****************************************************************************
  * NormScan : This IDCT uses reordered coeffs, so we patch the scan table
  *****************************************************************************/
@@ -750,7 +718,6 @@ void idct_block_add_altivec (vector_s16_t * block, unsigned char * dest,
  *****************************************************************************/
 static void idct_getfunctions( function_list_t * p_function_list )
 {
-    p_function_list->pf_probe = idct_Probe;
 #define F p_function_list->functions.idct
     F.pf_idct_init = InitIDCT;
     F.pf_norm_scan = NormScan;
