@@ -2,7 +2,7 @@
  * idctmmx.c : MMX IDCT module
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: idctmmx.c,v 1.20 2001/11/28 15:08:05 massiot Exp $
+ * $Id: idctmmx.c,v 1.23 2002/02/15 13:32:53 sam Exp $
  *
  * Authors: Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
  *          Michel Lespinasse <walken@zoy.org>
@@ -25,32 +25,19 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME idctmmx
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <stdlib.h>
 #include <string.h>
 
-#include "config.h"
-#include "common.h"
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"                                              /* TestCPU() */
+#include <videolan/vlc.h>
 
 #include "mmx.h"
 
 #include "idct.h"
 #include "block_mmx.h"
 
-#include "modules.h"
-#include "modules_export.h"
-
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
@@ -60,14 +47,14 @@ static void idct_getfunctions( function_list_t * p_function_list );
  * Build configuration tree.
  *****************************************************************************/
 MODULE_CONFIG_START
-ADD_WINDOW( "Configuration for MMX IDCT module" )
-    ADD_COMMENT( "Ha, ha -- nothing to configure yet" )
 MODULE_CONFIG_STOP
 
 MODULE_INIT_START
-    p_module->i_capabilities = MODULE_CAPABILITY_NULL
-                                | MODULE_CAPABILITY_IDCT;
-    p_module->psz_longname = "MMX IDCT module";
+    SET_DESCRIPTION( "MMX IDCT module" )
+    ADD_CAPABILITY( IDCT, 150 )
+    ADD_REQUIREMENT( MMX )
+    ADD_SHORTCUT( "mmx" )
+    ADD_SHORTCUT( "imdctmmx" )
 MODULE_INIT_STOP
 
 MODULE_ACTIVATE_START
@@ -79,25 +66,6 @@ MODULE_DEACTIVATE_STOP
 
 /* Following functions are local */
 
-/*****************************************************************************
- * idct_Probe: return a preference score
- *****************************************************************************/
-static int idct_Probe( probedata_t *p_data )
-{
-    if( !TestCPU( CPU_CAPABILITY_MMX ) )
-    {
-        return( 0 );
-    }
-
-    if( TestMethod( IDCT_METHOD_VAR, "idctmmx" )
-         || TestMethod( IDCT_METHOD_VAR, "mmx" ) )
-    {
-        return( 999 );
-    }
-
-    return( 150 );
-}
-
 /*****************************************************************************
  * NormScan : This IDCT uses reordered coeffs, so we patch the scan table
  *****************************************************************************/
