@@ -1,10 +1,13 @@
 /*****************************************************************************
- * macosx.h : MacOS X plugin for vlc
+ * macosx.h: MacOS X plugin for vlc
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $$
+ * $Id: macosx.h,v 1.5 2002/02/18 01:34:44 jlj Exp $
  *
- * Authors: Florian G. Pflug <fgp@phlo.org>
+ * Authors: Colin Delacroix <colin@zoy.org>
+ *          Eugenio Jarosiewicz <ej0@cise.ufl.edu>
+ *          Florian G. Pflug <fgp@phlo.org>
+ *          Jon Lech Johansen <jon-vl@nanocrew.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,26 +24,55 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
+#import <Cocoa/Cocoa.h>
+#import <QuickTime/QuickTime.h>
+
+#include "vout_window.h"
+#include "vout_qdview.h"
+
 /*****************************************************************************
- * The vout struct as access from both, the output and the interface module
+ * intf_sys_t: description and status of the interface
  *****************************************************************************/
-#include <QuickTime/QuickTime.h>
+typedef struct intf_sys_s
+{
+    NSPort *o_port;
+    NSAutoreleasePool *o_pool;
+} intf_sys_t;
 
-#define OSX_INTF_VOUT_QDPORT_CHANGE	0x0001
-#define OSX_INTF_VOUT_SIZE_CHANGE	0x0002
-#define OSX_VOUT_INTF_REQUEST_QDPORT	0x0004
-#define OSX_VOUT_INTF_RELEASE_QDPORT	0x0008
+/*****************************************************************************
+ * vout_sys_t: MacOS X video output method descriptor
+ *****************************************************************************/
+typedef struct vout_sys_s
+{
+    VLCWindow *o_window;
 
-/* This struct is included as the _FIRST_ member in vout_sys_t */
-/* That way the interface can cast the vout_sys_t to osx_com_t */
-/* and doesn't need the definition of vout_sys_t */
-#ifndef OSX_COM_TYPE
-    #define OSX_COM_TYPE osx_com_t
-    #define OSX_COM_STRUCT osx_com_s
-#endif
-typedef struct OSX_COM_STRUCT {
-    vlc_mutex_t lock ;
-    unsigned int i_changes ;
-    
-    CGrafPtr p_qdport ;
-} OSX_COM_TYPE ;
+    NSRect s_rect;
+    int b_pos_saved;
+
+    boolean_t b_mouse_moved;
+    boolean_t b_mouse_pointer_visible;
+    mtime_t i_time_mouse_last_moved;
+
+    CodecType i_codec;
+    CGrafPtr p_qdport;
+    ImageSequence i_seq;
+    MatrixRecordPtr p_matrix;
+    DecompressorComponent img_dc;
+    ImageDescriptionHandle h_img_descr;
+} vout_sys_t;
+
+/*****************************************************************************
+ * vout_req_t: MacOS X video output request 
+ *****************************************************************************/
+#define VOUT_REQ_CREATE_WINDOW  0x00000001
+#define VOUT_REQ_DESTROY_WINDOW 0x00000002
+
+typedef struct vout_req_s
+{
+    int i_type;
+    int i_result;
+
+    NSConditionLock *o_lock;
+
+    struct vout_thread_s *p_vout;
+} vout_req_t;
