@@ -2,7 +2,7 @@
  * vpar_blocks.c : blocks parsing
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: vpar_blocks.c,v 1.2 2001/11/28 15:08:05 massiot Exp $
+ * $Id: vpar_blocks.c,v 1.7 2002/03/17 17:00:38 sam Exp $
  *
  * Authors: Michel Lespinasse <walken@zoy.org>
  *          Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
@@ -26,24 +26,16 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <string.h>                                                /* memset */
 
-#include "config.h"
-#include "common.h"
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
+#include <videolan/vlc.h>
 
 #include "video.h"
 #include "video_output.h"
 
-#include "modules_export.h"
 #include "stream_control.h"
 #include "input_ext-dec.h"
 
-
 #include "vdec_ext-plugins.h"
 #include "vpar_pool.h"
 #include "video_parser.h"
@@ -543,7 +535,7 @@ coeff_2:
             if( i_coeff >= 64 )
             {
                 /* Illegal, but needed to avoid overflow */
-                intf_WarnMsg( 2, "MPEG2NonIntra coeff is out of bound" );
+                intf_WarnMsg( 2, "vpar warning: MPEG2NonIntra coeff is out of bound" );
                 p_vpar->picture.b_error = 1;
                 break;
             }
@@ -597,7 +589,7 @@ coeff_2:
             }
         }
 
-        intf_WarnMsg( 2, "MPEG2NonIntra coeff is out of bound" );
+        intf_WarnMsg( 2, "vpar warning: MPEG2NonIntra coeff is out of bound" );
         p_vpar->picture.b_error = 1;
         break;
     }
@@ -701,7 +693,7 @@ store_coeff:
             if( i_coeff >= 64 )
             {
                 /* Illegal, but needed to avoid overflow */
-                intf_WarnMsg( 2, "MPEG1Intra coeff is out of bound" );
+                intf_WarnMsg( 2, "vpar warning: MPEG1Intra coeff is out of bound" );
                 p_vpar->picture.b_error = 1;
                 break;
             }
@@ -763,7 +755,7 @@ store_coeff:
             }
         }
 
-        intf_WarnMsg( 2, "MPEG1Intra coeff is out of bound" );
+        intf_WarnMsg( 2, "vpar warning: MPEG1Intra coeff is out of bound" );
         p_vpar->picture.b_error = 1;
         break;
     }
@@ -856,7 +848,7 @@ coeff_2:
             if( i_coeff >= 64 )
             {
                 /* Illegal, but needed to avoid overflow */
-                intf_WarnMsg( 2, "MPEG1NonIntra coeff is out of bound" );
+                intf_WarnMsg( 2, "vpar warning: MPEG1NonIntra coeff is out of bound" );
                 p_vpar->picture.b_error = 1;
                 break;
             }
@@ -918,7 +910,7 @@ coeff_2:
             }
         }
 
-        intf_WarnMsg( 2, "MPEG1NonIntra coeff is out of bound" );
+        intf_WarnMsg( 2, "vpar warning: MPEG1NonIntra coeff is out of bound" );
         p_vpar->picture.b_error = 1;
         break;
     }
@@ -1822,20 +1814,20 @@ static __inline__ void ParseSlice( vpar_thread_t * p_vpar,
         if( i_coding_type != I_CODING_TYPE || p_vpar->picture.b_concealment_mv )
         {
             p_f_motion->pppi_ref[1][0] =
-                pp_forward_ref[1]->p_y + i_offset * 4 + i_width;
+                pp_forward_ref[1]->Y_PIXELS + i_offset * 4 + i_width;
             p_f_motion->pppi_ref[1][1] =
-                pp_forward_ref[1]->p_u + i_chroma_tmp;
+                pp_forward_ref[1]->U_PIXELS + i_chroma_tmp;
             p_f_motion->pppi_ref[1][2] =
-                pp_forward_ref[1]->p_v + i_chroma_tmp;
+                pp_forward_ref[1]->V_PIXELS + i_chroma_tmp;
         }
         if( i_coding_type == B_CODING_TYPE )
         {
             p_b_motion->pppi_ref[1][0] =
-                p_vpar->sequence.p_backward->p_y + i_offset * 4 + i_width;
+                p_vpar->sequence.p_backward->Y_PIXELS + i_offset * 4 + i_width;
             p_b_motion->pppi_ref[1][1] =
-                p_vpar->sequence.p_backward->p_u + i_chroma_tmp;
+                p_vpar->sequence.p_backward->U_PIXELS + i_chroma_tmp;
             p_b_motion->pppi_ref[1][2] =
-                p_vpar->sequence.p_backward->p_v + i_chroma_tmp;
+                p_vpar->sequence.p_backward->V_PIXELS + i_chroma_tmp;
         }
     }
 
@@ -1844,9 +1836,9 @@ static __inline__ void ParseSlice( vpar_thread_t * p_vpar,
                         * (2 - p_vpar->sequence.b_chroma_h_subsampled);
     if( i_coding_type != I_CODING_TYPE || p_vpar->picture.b_concealment_mv )
     {
-        p_f_motion->pppi_ref[0][0] = pp_forward_ref[0]->p_y + i_offset * 4;
-        p_f_motion->pppi_ref[0][1] = pp_forward_ref[0]->p_u + i_chroma_tmp;
-        p_f_motion->pppi_ref[0][2] = pp_forward_ref[0]->p_v + i_chroma_tmp;
+        p_f_motion->pppi_ref[0][0] = pp_forward_ref[0]->Y_PIXELS + i_offset * 4;
+        p_f_motion->pppi_ref[0][1] = pp_forward_ref[0]->U_PIXELS + i_chroma_tmp;
+        p_f_motion->pppi_ref[0][2] = pp_forward_ref[0]->V_PIXELS + i_chroma_tmp;
         p_f_motion->pi_f_code[0] = p_vpar->picture.ppi_f_code[0][0];
         p_f_motion->pi_f_code[1] = p_vpar->picture.ppi_f_code[0][1];
         p_f_motion->ppi_pmv[0][0] = p_f_motion->ppi_pmv[0][1] = 0;
@@ -1855,12 +1847,12 @@ static __inline__ void ParseSlice( vpar_thread_t * p_vpar,
 
     if( i_coding_type == B_CODING_TYPE )
     {
-        p_b_motion->pppi_ref[0][0] = p_vpar->sequence.p_backward->p_y
+        p_b_motion->pppi_ref[0][0] = p_vpar->sequence.p_backward->Y_PIXELS
                                         + i_offset * 4;
-        p_b_motion->pppi_ref[0][1] = p_vpar->sequence.p_backward->p_u
-            + i_chroma_tmp;
-        p_b_motion->pppi_ref[0][2] = p_vpar->sequence.p_backward->p_v
-            + i_chroma_tmp;
+        p_b_motion->pppi_ref[0][1] = p_vpar->sequence.p_backward->U_PIXELS
+                                        + i_chroma_tmp;
+        p_b_motion->pppi_ref[0][2] = p_vpar->sequence.p_backward->V_PIXELS
+                                        + i_chroma_tmp;
         p_b_motion->pi_f_code[0] = p_vpar->picture.ppi_f_code[1][0];
         p_b_motion->pi_f_code[1] = p_vpar->picture.ppi_f_code[1][1];
         p_b_motion->ppi_pmv[0][0] = p_b_motion->ppi_pmv[0][1] = 0;
@@ -1868,9 +1860,9 @@ static __inline__ void ParseSlice( vpar_thread_t * p_vpar,
     }
 
     /* Initialize destination pointers. */
-    p_dest[0] = p_vpar->picture.p_picture->p_y + i_offset * 4;
-    p_dest[1] = p_vpar->picture.p_picture->p_u + i_chroma_tmp;
-    p_dest[2] = p_vpar->picture.p_picture->p_v + i_chroma_tmp;
+    p_dest[0] = p_vpar->picture.p_picture->Y_PIXELS + i_offset * 4;
+    p_dest[1] = p_vpar->picture.p_picture->U_PIXELS + i_chroma_tmp;
+    p_dest[2] = p_vpar->picture.p_picture->V_PIXELS + i_chroma_tmp;
 
     if( i_structure == BOTTOM_FIELD )
     {
