@@ -20,14 +20,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME qnx 
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <errno.h>                                                 /* ENOMEM */
 #include <stdlib.h>                                                /* free() */
 #include <string.h>                                            /* strerror() */
@@ -37,20 +32,13 @@
 #include <photon/PtLabel.h>
 #include <photon/PdDirect.h>
 
-#include "common.h"
-#include "intf_msg.h"
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"
+#include <videolan/vlc.h>
 
 #include "video.h"
 #include "video_output.h"
 
 #include "interface.h"
 
-#include "modules.h"
-#include "modules_export.h"
-
 /*****************************************************************************
  * vout_sys_t: video output QNX method descriptor
  *****************************************************************************
@@ -101,7 +89,6 @@ typedef struct vout_sys_s
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static int  vout_Probe     ( probedata_t *p_data );
 static int  vout_Create    ( struct vout_thread_s * );
 static int  vout_Init      ( struct vout_thread_s * );
 static void vout_End       ( struct vout_thread_s * );
@@ -119,7 +106,6 @@ static int  QNXDestroyWnd  ( struct vout_thread_s * );
  *****************************************************************************/
 void _M( vout_getfunctions )( function_list_t * p_function_list )
 {
-    p_function_list->pf_probe = vout_Probe;
     p_function_list->functions.vout.pf_create     = vout_Create;
     p_function_list->functions.vout.pf_init       = vout_Init;
     p_function_list->functions.vout.pf_end        = vout_End;
@@ -129,22 +115,6 @@ void _M( vout_getfunctions )( function_list_t * p_function_list )
     p_function_list->functions.vout.pf_setpalette = NULL;
 }
 
-/*****************************************************************************
- * vout_Probe: probe the video driver and return a score
- *****************************************************************************
- * This function tries to initialize SDL and returns a score to the
- * plugin manager so that it can select the best plugin.
- *****************************************************************************/
-static int vout_Probe( probedata_t *p_data )
-{
-    if( TestMethod( VOUT_METHOD_VAR, "qnx" ) )
-    {
-        return( 999 );
-    }
-
-    return( 100 );
-}
-
 /*****************************************************************************
  * vout_Create: allocate QNX video thread output method
  *****************************************************************************
@@ -172,14 +142,12 @@ static int vout_Create( vout_thread_t *p_vout )
     memset( p_vout->p_sys, 0, sizeof( vout_sys_t ) );
 
     p_vout->b_fullscreen = 
-        main_GetIntVariable( VOUT_FULLSCREEN_VAR, VOUT_FULLSCREEN_DEFAULT );
+        config_GetIntVariable( "fullscreen" );
     p_vout->p_sys->i_mode = 
-        main_GetIntVariable( VOUT_OVERLAY_VAR, VOUT_OVERLAY_DEFAULT ) ?
-        MODE_VIDEO_OVERLAY : MODE_NORMAL_MEM;
-    p_vout->p_sys->dim.w =
-        main_GetIntVariable( VOUT_WIDTH_VAR, VOUT_WIDTH_DEFAULT );
-    p_vout->p_sys->dim.h =
-        main_GetIntVariable( VOUT_HEIGHT_VAR, VOUT_HEIGHT_DEFAULT );
+        config_GetIntVariable( "nooverlay" ) ?
+        MODE_NORMAL_MEM : MODE_VIDEO_OVERLAY;
+    p_vout->p_sys->dim.w = p_vout->i_window_width;
+    p_vout->p_sys->dim.h = p_vout->i_window_height;
 
     /* init display and create window */
     if( QNXInitDisplay( p_vout ) || QNXCreateWnd( p_vout ) )
@@ -199,11 +167,15 @@ static int vout_Create( vout_thread_t *p_vout )
  *****************************************************************************/
 static int vout_Init( vout_thread_t *p_vout )
 {
-    if( p_vout->p_sys->i_mode == MODE_NORMAL_MEM ||
-        p_vout->p_sys->i_mode == MODE_SHARED_MEM )
-    {
-        /* create images for [shared] memory blit */
+    /* These variables are only used if we are in overlay mode */
+    int i_ret;
+    PgScalerProps_t props;
 
+    switch( p_vout->p_sys->i_mode )
+    {
+    case MODE_NORMAL_MEM:
+    case MODE_SHARED_MEM:
+        /* create images for [shared] memory blit */
         if( !( p_vout->p_sys->p_image[0] = PhCreateImage( NULL,
                     p_vout->p_sys->dim.w, p_vout->p_sys->dim.h,
                     p_vout->p_sys->i_img_type, NULL, 0,
@@ -227,11 +199,10 @@ static int vout_Init( vout_thread_t *p_vout )
         p_vout->i_bytes_per_line = p_vout->p_sys->p_image[0]->bpl;
         p_vout->pf_setbuffers( p_vout, p_vout->p_sys->p_image[0]->image,
                                p_vout->p_sys->p_image[1]->image );
-    }
-    else if( p_vout->p_sys->i_mode == MODE_VIDEO_MEM )
-    {
-        /* create offscreen contexts for video memory blit */
+        break;
 
+    case MODE_VIDEO_MEM:
+        /* create offscreen contexts for video memory blit */
         if( ( p_vout->p_sys->p_ctx[0] = PdCreateOffscreenContext( 0,
                         p_vout->p_sys->dim.w, p_vout->p_sys->dim.h,
                         Pg_OSC_MEM_PAGE_ALIGN ) ) == NULL )
@@ -272,12 +243,8 @@ static int vout_Init( vout_thread_t *p_vout )
             p_vout->i_bytes_per_line * p_vout->p_sys->dim.h );
         p_vout->pf_setbuffers( p_vout, p_vout->p_sys->p_buf[0],
                                p_vout->p_sys->p_buf[1] );
-    }
-    else if( p_vout->p_sys->i_mode == MODE_VIDEO_OVERLAY )
-    {
-        int i_ret;
-        PgScalerProps_t props;
 
+    case MODE_VIDEO_OVERLAY:
         props.size   = sizeof( props );
         props.format = p_vout->p_sys->i_vc_format; 
         props.flags  = Pg_SCALER_PROP_SCALER_ENABLE |
@@ -363,6 +330,11 @@ static int vout_Init( vout_thread_t *p_vout )
             p_vout->pf_setbuffers( p_vout,
                 p_vout->p_sys->p_vc_y[0], p_vout->p_sys->p_vc_y[1] );
         }
+        break;
+
+    default:
+        /* This shouldn't happen ! */
+        break;
     }
 
     return( 0 );
@@ -502,12 +474,6 @@ static int vout_Manage( vout_thread_t *p_vout )
                         {
                             network_ChannelJoin( i_key );
                         }
-                        else if( intf_ProcessKey( p_main->p_intf,
-                                                    (char) i_key ) )
-                        {
-                            intf_DbgMsg( "vout: unhandled key '%c' (%i)",
-                                         (char) i_key, i_key );
-                        }
                         break;
                     }
                 }
@@ -524,8 +490,6 @@ static int vout_Manage( vout_thread_t *p_vout )
     {
         PhDim_t dim;
 
-        intf_DbgMsg( "vout: changing full-screen status" );
-
         p_vout->b_fullscreen = !p_vout->b_fullscreen;
         p_vout->i_changes &= ~VOUT_FULLSCREEN_CHANGE;
 
@@ -566,7 +530,6 @@ static int vout_Manage( vout_thread_t *p_vout )
      */
     if( p_vout->i_changes & VOUT_SIZE_CHANGE )
     {
-        intf_DbgMsg( "vout: resizing window" );
         p_vout->i_changes &= ~VOUT_SIZE_CHANGE;
 
         if( p_vout->p_sys->i_mode != MODE_VIDEO_OVERLAY )
@@ -592,8 +555,6 @@ static int vout_Manage( vout_thread_t *p_vout )
      */
     if( b_repos && p_vout->p_sys->i_mode == MODE_VIDEO_OVERLAY )
     {
-        intf_DbgMsg( "vout: moving video channel" );
-
         vout_End( p_vout );
         if( vout_Init( p_vout ) )
         {
@@ -696,7 +657,6 @@ static int QNXInitDisplay( p_vout_thread_t p_vout )
         hwcaps.currently_available_video_ram >= 
         ( ( minfo.width * minfo.height * minfo.bits_per_pixel ) / 8 ) )
     {
-        intf_DbgMsg( "vout: using video ram" );
         p_vout->p_sys->i_mode = MODE_VIDEO_MEM;
     }
 
@@ -724,23 +684,23 @@ static int QNXInitDisplay( p_vout_thread_t p_vout )
     switch( p_vout->i_screen_depth )
     {
         case 15:
-            p_vout->i_red_mask   = 0x7c00;
-            p_vout->i_green_mask = 0x03e0;
-            p_vout->i_blue_mask  = 0x001f;
+            p_vout->output.i_rmask = 0x7c00;
+            p_vout->output.i_gmask = 0x03e0;
+            p_vout->output.i_bmask = 0x001f;
             break;
 
         case 16:
-            p_vout->i_red_mask   = 0xf800;
-            p_vout->i_green_mask = 0x07e0;
-            p_vout->i_blue_mask  = 0x001f;
+            p_vout->output.i_rmask = 0xf800;
+            p_vout->output.i_gmask = 0x07e0;
+            p_vout->output.i_bmask = 0x001f;
             break;
 
         case 24:
         case 32:
         default:
-            p_vout->i_red_mask   = 0xff0000;
-            p_vout->i_green_mask = 0x00ff00;
-            p_vout->i_blue_mask  = 0x0000ff;
+            p_vout->output.i_rmask = 0xff0000;
+            p_vout->output.i_gmask = 0x00ff00;
+            p_vout->output.i_bmask = 0x0000ff;
             break;
     }
 
