@@ -2,7 +2,7 @@
  * aout_sdl.c : audio sdl functions library
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: aout_sdl.c,v 1.20.2.2 2001/12/18 02:19:35 sam Exp $
+ * $Id: aout_sdl.c,v 1.27 2002/02/24 22:06:50 sam Exp $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -23,14 +23,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define MODULE_NAME sdl
-#include "modules_inner.h"
-
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include "defs.h"
-
 #include <errno.h>                                                 /* ENOMEM */
 #include <fcntl.h>                                       /* open(), O_WRONLY */
 #include <string.h>                                            /* strerror() */
@@ -38,19 +33,12 @@
 #include <stdio.h>                                           /* "intf_msg.h" */
 #include <stdlib.h>                            /* calloc(), malloc(), free() */
 
+#include <videolan/vlc.h>
+
 #include SDL_INCLUDE_FILE
 
-#include "common.h"                                     /* boolean_t, byte_t */
-#include "intf_msg.h"                        /* intf_DbgMsg(), intf_ErrMsg() */
-#include "threads.h"
-#include "mtime.h"
-#include "tests.h"
-
 #include "audio_output.h"                                   /* aout_thread_t */
 
-#include "modules.h"
-#include "modules_export.h"
-
 /*****************************************************************************
  * aout_sys_t: dsp audio output method descriptor
  *****************************************************************************
@@ -73,10 +61,9 @@ typedef struct aout_sys_s
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
-static int     aout_Probe       ( probedata_t *p_data );
 static int     aout_Open        ( aout_thread_t *p_aout );
 static int     aout_SetFormat   ( aout_thread_t *p_aout );
-static long    aout_GetBufInfo  ( aout_thread_t *p_aout, long l_buffer_info );
+static int     aout_GetBufInfo  ( aout_thread_t *p_aout, int i_buffer_info );
 static void    aout_Play        ( aout_thread_t *p_aout,
                                   byte_t *buffer, int i_size );
 static void    aout_Close       ( aout_thread_t *p_aout );
@@ -89,7 +76,6 @@ static void    aout_SDLCallback ( void *userdata, Uint8 *stream, int len );
  *****************************************************************************/
 void _M( aout_getfunctions )( function_list_t * p_function_list )
 {
-    p_function_list->pf_probe = aout_Probe;
     p_function_list->functions.aout.pf_open = aout_Open;
     p_function_list->functions.aout.pf_setformat = aout_SetFormat;
     p_function_list->functions.aout.pf_getbufinfo = aout_GetBufInfo;
@@ -97,59 +83,6 @@ void _M( aout_getfunctions )( function_list_t * p_function_list )
     p_function_list->functions.aout.pf_close = aout_Close;
 }
 
-/*****************************************************************************
- * aout_Probe: probe the audio device and return a score
- *****************************************************************************
- * This function tries to initialize SDL audio and returns a score to the
- * plugin manager so that it can select the best plugin.
- *****************************************************************************/
-static int aout_Probe( probedata_t *p_data )
-{
-#if 0
-    SDL_AudioSpec desired, obtained;
-#endif
-
-    if( SDL_WasInit( SDL_INIT_AUDIO ) != 0 )
-    {
-        return( 0 );
-    }
-
-#if 0
-    /* Start AudioSDL */
-    if( SDL_Init(SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE) != 0 )
-    {
-        intf_DbgMsg( "aout: SDL_Init failed (%s)", SDL_GetError() );
-        return( 0 );
-    }
-
-    desired.freq       = 11025;                                 /* frequency */
-    desired.format     = AUDIO_U8;                        /* unsigned 8 bits */
-    desired.channels   = 2;                                          /* mono */
-    desired.callback   = aout_SDLCallback;    /* callback function mandatory */
-    desired.userdata   = NULL;                     /* null parm for callback */
-    desired.samples    = 4096;
-
-    /* If we were unable to open the device, there is no way we can use
-     * the plugin. Return a score of 0. */
-    if( SDL_OpenAudio( &desired, &obtained ) < 0 )
-    {
-        intf_DbgMsg( "aout: SDL_OpenAudio failed (%s)", SDL_GetError() );
-        return( 0 );
-    }
-
-    /* Otherwise, there are good chances we can use this plugin, return 100. */
-    intf_DbgMsg( "aout: SDL_OpenAudio successfully run" );
-    SDL_CloseAudio();
-#endif
-
-    if( TestMethod( AOUT_METHOD_VAR, "sdl" ) )
-    {
-        return( 999 );
-    }
-
-    return( 40 );
-}
-
 /*****************************************************************************
  * aout_Open: open the audio device
  *****************************************************************************
@@ -159,7 +92,11 @@ static int aout_Probe( probedata_t *p_data )
 static int aout_Open( aout_thread_t *p_aout )
 {
     SDL_AudioSpec desired;
-    int i_channels = p_aout->b_stereo ? 2 : 1;
+
+    if( SDL_WasInit( SDL_INIT_AUDIO ) != 0 )
+    {
+        return( 1 );
+    }
 
     /* Allocate structure */
     p_aout->p_sys = malloc( sizeof( aout_sys_t ) );
@@ -192,19 +129,12 @@ static int aout_Open( aout_thread_t *p_aout )
     p_aout->p_sys->audio_buf = malloc( OVERFLOWLIMIT );
 
     /* Initialize some variables */
-    p_aout->psz_device = 0;
-    p_aout->i_format   = AOUT_FORMAT_DEFAULT;
-    p_aout->i_channels = 1 + main_GetIntVariable( AOUT_STEREO_VAR,
-                                                  AOUT_STEREO_DEFAULT );
-    p_aout->l_rate     = main_GetIntVariable( AOUT_RATE_VAR,
-                                              AOUT_RATE_DEFAULT );
-
-    desired.freq =     p_aout->l_rate;
 
     /* TODO: write conversion beetween AOUT_FORMAT_DEFAULT
      * AND AUDIO* from SDL. */
+    desired.freq       = p_aout->i_rate;
     desired.format     = AUDIO_S16LSB;                     /* stereo 16 bits */
-    desired.channels   = i_channels;
+    desired.channels   = p_aout->i_channels;
     desired.callback   = aout_SDLCallback;
     desired.userdata   = p_aout->p_sys;
     desired.samples    = 1024;
@@ -217,6 +147,8 @@ static int aout_Open( aout_thread_t *p_aout )
     if( SDL_OpenAudio( &desired, NULL ) < 0 )
     {
         intf_ErrMsg( "aout error: SDL_OpenAudio failed (%s)", SDL_GetError() );
+        SDL_QuitSubSystem( SDL_INIT_AUDIO );
+        free( p_aout->p_sys );
         return( -1 );
     }
 
@@ -238,12 +170,11 @@ static int aout_SetFormat( aout_thread_t *p_aout )
 {
     /* TODO: finish and clean this */
     SDL_AudioSpec desired;
-    int i_stereo = p_aout->b_stereo ? 2 : 1;
 
     /*i_format = p_aout->i_format;*/
-    desired.freq       = p_aout->l_rate;             /* Set the output rate */
+    desired.freq       = p_aout->i_rate;             /* Set the output rate */
     desired.format     = AUDIO_S16LSB;                    /* stereo 16 bits */
-    desired.channels   = i_stereo;
+    desired.channels   = p_aout->i_channels;
     desired.callback   = aout_SDLCallback;
     desired.userdata   = p_aout->p_sys;
     desired.samples    = 2048;
@@ -255,16 +186,12 @@ static int aout_SetFormat( aout_thread_t *p_aout )
     if( SDL_OpenAudio( &desired, NULL ) < 0 )
     {
         p_aout->p_sys->b_active = 0;
-        SDL_QuitSubSystem( SDL_INIT_AUDIO );
-        free( p_aout->p_sys );
         return( -1 );
     }
 
     p_aout->p_sys->b_active = 1;
     SDL_PauseAudio( 0 );
 
-    p_aout->i_latency = 0;
-    
     return( 0 );
 }
 
@@ -272,16 +199,16 @@ static int aout_SetFormat( aout_thread_t *p_aout )
  * aout_GetBufInfo: buffer status query
  *****************************************************************************
  * returns the number of bytes in the audio buffer compared to the size of
- * l_buffer_limit...
+ * i_buffer_limit...
  *****************************************************************************/
-static long aout_GetBufInfo( aout_thread_t *p_aout, long l_buffer_limit )
+static int aout_GetBufInfo( aout_thread_t *p_aout, int i_buffer_limit )
 {
-    if(l_buffer_limit > p_aout->p_sys->i_audio_end)
+    if(i_buffer_limit > p_aout->p_sys->i_audio_end)
     {
         /* returning 0 here juste gives awful sound in the speakers :/ */
-        return( l_buffer_limit );
+        return( i_buffer_limit );
     }
-    return( p_aout->p_sys->i_audio_end - l_buffer_limit);
+    return( p_aout->p_sys->i_audio_end - i_buffer_limit);
 }
 
 /*****************************************************************************
