@@ -2,9 +2,10 @@
  * win32_specific.c: Win32 specific features 
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: win32_specific.c,v 1.3 2001/11/28 15:08:06 massiot Exp $
+ * $Id: win32_specific.c,v 1.6 2002/04/02 23:43:57 gbazin Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
+ *          Gildas Bazin <gbazin@netcourrier.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,26 +21,37 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
-#include "defs.h"
-
+#include <errno.h>                                                 /* ENOMEM */
 #include <string.h>                                              /* strdup() */
 #include <stdlib.h>                                                /* free() */
 #include <fcntl.h>
 
 #include <winsock2.h>
 
-#include "config.h"
-#include "common.h"
-
-#include "win32_specific.h"
+#include <videolan/vlc.h>
 
 /*****************************************************************************
- * system_Init: initialize winsock.
+ * system_Init: initialize winsock and misc other things.
  *****************************************************************************/
 void system_Init( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
 {
     WSADATA Data;
     int i_err;
+    HINSTANCE hInstLib;
+
+    /* Allocate structure */
+    p_main_sys = malloc( sizeof( main_sys_t ) );
+    if( p_main_sys == NULL )
+    {
+        intf_ErrMsg( "init error: can't create p_main_sys (%s)",
+		     strerror(ENOMEM) );
+        exit(-1);
+    }
+
+    /* dynamically get the address of SignalObjectAndWait */
+    hInstLib = LoadLibrary( "kernel32" );
+    p_main_sys->SignalObjectAndWait =
+        (SIGNALOBJECTANDWAIT)GetProcAddress( hInstLib, "SignalObjectAndWait" );
 
     /* WinSock Library Init. */
     i_err = WSAStartup( MAKEWORD( 1, 1 ), &Data );
@@ -52,6 +64,14 @@ void system_Init( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
     _fmode = _O_BINARY;  /* sets the default file-translation mode */
 }
 
+/*****************************************************************************
+ * system_Configure: check for system specific configuration options.
+ *****************************************************************************/
+void system_Configure( void )
+{
+    p_main_sys->b_fast_pthread = config_GetIntVariable( "fast_pthread" );
+}
+
 /*****************************************************************************
  * system_End: terminate winsock.
  *****************************************************************************/
@@ -59,4 +79,3 @@ void system_End( void )
 {
     WSACleanup();
 }
-
